/************************************
 Miscellaneous functions
 (c) 1999 Jeremy Wise
 GnomeICU
*************************************/

/*** GnomeICU header files ***/
#include "common.h"

/*** Toplevel header files ***/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h>
#include <fcntl.h>
#include <time.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>
#ifdef HAVE_SOCKS5
#define SOCKS
#include <socks.h>
#endif

typedef struct
{
	gpointer data;
	int i;
} dataandint;
      
GtkWidget *found_list;

int start_new_info = TRUE;

/*** Local function declarations ***/
static void icq_allow_auth( GtkWidget *widget, gpointer data );
static void icq_accept_chat( GtkWidget *widget, GSList *contact );
static void icq_refuse_chat( GtkWidget *widget, GSList *contact );
static void icq_accept_file( GtkWidget *widget, GSList *contact );
static void icq_refuse_file( GtkWidget *widget, GSList *contact );
static void icq_refresh_list( GtkWidget *widget, gpointer data );
static void clear_i_window( GtkWidget *wigdet, gpointer data );
static void grab_history( GtkWidget *widget, gpointer data );
static void get_contact_list( GtkWidget *widget, gpointer data );
static void remove_user_question( GtkWidget *widget, gpointer data );
static void cb_menu_position (GtkMenu *menu, gint *x, gint *y, gpointer user_data);
static void reply_button_cb( GtkWidget *widget, gpointer data );
static gchar *stripr( gchar *str );
static void add_user_to_list( GtkWidget *widget, gpointer data );

gint animate_ref = 0;

/*** Global functions ***/
void animate_on( void )
{
	++animate_ref;

	/* If the eyes are already spinning, no need to start them again */
	if( animate_ref != 1 )
		return;

	gtk_widget_hide( eyes_still );
	gtk_widget_show( eyes_animated );
	gnome_animator_set_loop_type( GNOME_ANIMATOR( eyes_animated ),
	                              GNOME_ANIMATOR_LOOP_RESTART );
	gnome_animator_start( GNOME_ANIMATOR( eyes_animated ) );
}

void animate_off( void )
{
	--animate_ref;

	if( animate_ref < 0 )
		animate_ref = 0;

	/* If we're not back down to zero yet, don't stop the eyes */
	if( animate_ref )
		return;

	gnome_animator_stop( GNOME_ANIMATOR( eyes_animated ) );
	gtk_widget_hide( eyes_animated );
	gtk_widget_show( eyes_still );
}

void icq_quit( GtkWidget *widget, gpointer data )
{
	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "icq_quit\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( ((CONTACT_PTR)contact->data)->sok > 0 )
			close( ((CONTACT_PTR)contact->data)->sok );
		if( ((CONTACT_PTR)contact->data)->chat_sok > 0 )
			close( ((CONTACT_PTR)contact->data)->chat_sok );
		contact = contact->next;
	}

	if( MainData->window != NULL && GTK_WIDGET_VISIBLE( MainData->window ) )
		gdk_window_get_size( GTK_WIDGET( MainData->window )->window,
		                     &WindowWidth,
		                     &WindowHeight );

	Save_RC();
	gtk_main_quit();
}

void icq_refresh( gpointer data, int sok, GdkInputCondition cond )
{
	static struct timeval tv;
	static fd_set readfds;
	int status;

	tv.tv_sec = 2;
	tv.tv_usec = 500000;

#ifdef TRACE_FUNCTION
	g_print( "icq_refresh\n" );
#endif

	FD_ZERO(&readfds);
	FD_SET( MainData->sok, &readfds);

	FD_SET(STDIN, &readfds);

	/* Don't care about writefds and exceptfds: */
	select( MainData->sok+1, &readfds, NULL, NULL, &tv);

	if (FD_ISSET( MainData->sok, &readfds))
	{
		Handle_Server_Response();
	}

	if( ( status = Show_Quick_Status() ) > 0 )
	{
		if( status == 1 )
			gnomeicu_event( EV_USERON );
		if( status == 2 )
			gnomeicu_event( EV_USEROFF );
	}
}

void icq_msgbox( STORED_MESSAGE_PTR message_text, UIN_T uin )
{
	GtkWidget *window = NULL;
	GtkWidget *message = NULL;
	GtkWidget *button;
	GtkWidget *table;
	GtkWidget *mainbox = NULL;
	GtkWidget *box;
	GtkWidget *menuitem;
	GtkWidget *entry1;
	GtkWidget *entry2;
	GtkWidget *dialog_vbox1;
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *hbox1;
	GtkWidget *hbox2;
	GtkWidget *hbox3;
	GtkWidget *label1;
	GtkWidget *label2;
	GtkWidget *label3;
	GtkWidget *label4;
	GtkWidget *label5;
	GtkWidget *label6;
	GtkWidget *fromfield;
	GtkWidget *timefield;
	GtkWidget *scrolledwindow1;
	GtkWidget *text;
	GtkWidget *optionmenu;
	GtkWidget *optionmenu_menu;
	GtkWidget *scrolledwindow2;
	GtkWidget *respond_text;
	GtkWidget *dialog_action_area1;
	GtkWidget *button1;
	GtkWidget *button2;
	GtkWidget *button3;
	GtkWidget *button4;
	GtkWidget *button5;

	gchar *new_str;

	static GtkWidget *allow_button;

	char *msgtime = NULL;

	int cx;
	char *nick;
	char *ptr, *ptr2, *ptr3, *buf[2];

	gchar *sender = NULL;

	GSList *contact, *contact2;

	int picked_user;

	GtkWidget *contact_list_clist;

	gchar *curl, *cdesc;
	GtkWidget *url, *desc;

#ifdef TRACE_FUNCTION
	g_print( "icq_msgbox\n" );
#endif
	
	contact = Contacts;

	while( contact != NULL )
	{
		if( uin == ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	nick = ((CONTACT_PTR)contact->data)->nick;

	while( contact != NULL )
	{
		if( uin == ((CONTACT_PTR)contact->data)->uin )
		{
			if( g_slist_length( ((CONTACT_PTR)contact->data)->stored_messages ) == 1 )
			{
				((CONTACT_PTR)contact->data)->icon_p = GetIcon_p( ((CONTACT_PTR)contact->data)->status );
				((CONTACT_PTR)contact->data)->icon_b = GetIcon_b( ((CONTACT_PTR)contact->data)->status );
				((CONTACT_PTR)contact->data)->need_update = 1;
			}
			break;
		}
		contact = contact->next;
	}

/* Strip \r's out of text - \r's are for Windows people */
	new_str = stripr( ((STORED_MESSAGE_PTR)message_text)->message );
	g_free( ((STORED_MESSAGE_PTR)message_text)->message );
	((STORED_MESSAGE_PTR)message_text)->message = new_str;

/*** User has received a TEXT message ***/
	if( ((STORED_MESSAGE_PTR)message_text)->type == MESSAGE_TEXT )
	{
		window = gnome_dialog_new (_("GnomeICU: Message Received"), NULL);
		gtk_object_set_data (GTK_OBJECT (window), "window", window);
		gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);

		gtk_widget_set_usize( window, -1, 375 );

		dialog_vbox1 = GNOME_DIALOG (window)->vbox;
		gtk_object_set_data (GTK_OBJECT (window), "dialog_vbox1", dialog_vbox1);
		gtk_widget_show (dialog_vbox1);

		vbox1 = gtk_vbox_new (FALSE, 0);
		gtk_widget_ref (vbox1);
		gtk_object_set_data_full (GTK_OBJECT (window), "vbox1", vbox1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (vbox1);
		gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);
		gtk_container_set_border_width (GTK_CONTAINER (vbox1), 6);

		hbox1 = gtk_hbox_new (FALSE, 0);
		gtk_widget_ref (hbox1);
		gtk_object_set_data_full (GTK_OBJECT (window), "hbox1", hbox1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (hbox1);
		gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 4);

		label1 = gtk_label_new (_("Sender:"));
		gtk_widget_ref (label1);
		gtk_object_set_data_full (GTK_OBJECT (window), "label1", label1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label1);
		gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 4);

		fromfield = gtk_entry_new ();
		gtk_widget_ref (fromfield);
		gtk_object_set_data_full (GTK_OBJECT (window), "fromfield", fromfield,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (fromfield);
		gtk_box_pack_start (GTK_BOX (hbox1), fromfield, FALSE, TRUE, 0);
		gtk_entry_set_editable (GTK_ENTRY (fromfield), FALSE);
		gtk_entry_set_text( GTK_ENTRY( fromfield ), nick );

		label2 = gtk_label_new (_("Time/Date:"));
		gtk_widget_ref (label2);
		gtk_object_set_data_full (GTK_OBJECT (window), "label2", label2,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label2);
		gtk_box_pack_start (GTK_BOX (hbox1), label2, TRUE, TRUE, 4);
		gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_RIGHT);
		gtk_misc_set_alignment (GTK_MISC (label2), 1, 0.5);
		timefield = gtk_entry_new ();
		gtk_widget_ref (timefield);
		gtk_object_set_data_full (GTK_OBJECT (window), "timefield", timefield,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (timefield);
		gtk_box_pack_start (GTK_BOX (hbox1), timefield, FALSE, TRUE, 0);

		msgtime = g_strdup( message_text->message );
		cx = 0;
		while(msgtime[cx++] != '\n' );
		msgtime[cx] = 0;
		gtk_entry_set_text( GTK_ENTRY( timefield ), msgtime );
		g_free( msgtime );

		scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
		gtk_widget_ref (scrolledwindow1);
		gtk_object_set_data_full (GTK_OBJECT (window), "scrolledwindow1", scrolledwindow1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (scrolledwindow1);
		gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

		text = gtk_text_new (NULL, NULL);
		gtk_widget_ref (text);
		gtk_object_set_data_full (GTK_OBJECT (window), "text", text,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (text);
		gtk_text_set_word_wrap( GTK_TEXT( text ), TRUE );
		gtk_text_set_editable( GTK_TEXT( text ), FALSE );
		gtk_container_add (GTK_CONTAINER (scrolledwindow1), text);

		gtk_text_insert( GTK_TEXT( text ), NULL, NULL, NULL,
		                 ( strchr( ((STORED_MESSAGE_PTR)message_text)->message, '\n' ) + 1 ), -1 );

		hbox3 = gtk_hbox_new (FALSE, 0);
		gtk_widget_ref (hbox3);
		gtk_object_set_data_full (GTK_OBJECT (window), "hbox3", hbox3,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (hbox3);
		gtk_box_pack_start (GTK_BOX (vbox1), hbox3, FALSE, FALSE, 6);

		button5 = gtk_button_new_with_label( _("Reply >>") );
		gtk_widget_ref( button5 );
		gtk_object_set_data_full( GTK_OBJECT( window ), "button5", button5,
		                          (GtkDestroyNotify) gtk_widget_unref );
		gtk_widget_show( button5 );
		gtk_box_pack_end( GTK_BOX( hbox3 ), button5, FALSE, FALSE, 0 );

		gtk_signal_connect( GTK_OBJECT( button5 ), "clicked",
		                    GTK_SIGNAL_FUNC( reply_button_cb ),
		                    NULL );

		vbox2 = gtk_vbox_new( FALSE, 0 );
		gtk_widget_ref( vbox2 );
		gtk_object_set_data_full( GTK_OBJECT( window ), "vbox2", vbox2,
		                          (GtkDestroyNotify) gtk_widget_unref );
		gtk_box_pack_start( GTK_BOX( vbox1 ), vbox2, TRUE, TRUE, 0 );

		gtk_object_set_data( GTK_OBJECT( button5 ), "box", vbox2 );
		gtk_object_set_data( GTK_OBJECT( button5 ), "hbox", hbox3 );

		hbox2 = gtk_hbox_new (FALSE, 0);
		gtk_widget_ref (hbox2);
		gtk_object_set_data_full (GTK_OBJECT (window), "hbox2", hbox2,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (hbox2);
		gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, FALSE, 6);

		label3 = gtk_label_new (_("Reply to:"));
		gtk_widget_ref (label3);
		gtk_object_set_data_full (GTK_OBJECT (window), "label3", label3,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label3);
		gtk_box_pack_start (GTK_BOX (hbox2), label3, FALSE, FALSE, 0);
		gtk_misc_set_padding (GTK_MISC (label3), 4, 0);

		optionmenu = gtk_option_menu_new ();
		gtk_widget_ref (optionmenu);
		gtk_object_set_data_full (GTK_OBJECT (window), "optionmenu", optionmenu,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (optionmenu);
		gtk_box_pack_start (GTK_BOX (hbox2), optionmenu, FALSE, TRUE, 0);
		optionmenu_menu = gtk_menu_new ();

		contact2 = Contacts;
		cx = 0;
		picked_user = 0;
		while( contact2 != NULL )
		{
			menuitem = gtk_menu_item_new_with_label( ((CONTACT_PTR)contact2->data)->nick );
			gtk_widget_show (menuitem);
			gtk_object_set_data( GTK_OBJECT( menuitem ), "contact", contact2 );
			gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
			if( contact == contact2 )
				picked_user = cx;
			contact2 = contact2->next;
			cx++;
		}

		gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), optionmenu_menu);
		gtk_option_menu_set_history( GTK_OPTION_MENU( optionmenu ), picked_user );

		button4 = gtk_button_new_with_label (_("History"));
		gtk_widget_ref (button4);
		gtk_object_set_data_full (GTK_OBJECT (window), "button4", button4,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button4);
		gtk_box_pack_end (GTK_BOX (hbox2), button4, FALSE, FALSE, 0);
		gtk_object_set_data( GTK_OBJECT( button4 ), "menu", optionmenu_menu );

		scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
		gtk_widget_ref (scrolledwindow2);
		gtk_object_set_data_full (GTK_OBJECT (window), "scrolledwindow2", scrolledwindow2,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (scrolledwindow2);
		gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow2, TRUE, TRUE, 0);
		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

		respond_text = gtk_text_new (NULL, NULL);
		gtk_widget_ref (respond_text);
		gtk_object_set_data_full (GTK_OBJECT (window), "respond_text", respond_text,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (respond_text);
		gtk_text_set_word_wrap( GTK_TEXT( respond_text ), TRUE );
		gtk_container_add (GTK_CONTAINER (scrolledwindow2), respond_text);
		gtk_text_set_editable( GTK_TEXT( respond_text ), TRUE );

		dialog_action_area1 = GNOME_DIALOG (window)->action_area;
		gtk_object_set_data (GTK_OBJECT (window), "dialog_action_area1", dialog_action_area1);
		gtk_widget_show (dialog_action_area1);
		gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

		gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (window),
		                                        _("Reply"), GNOME_STOCK_PIXMAP_JUMP_TO);
		button1 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
		gtk_widget_ref (button1);
		gtk_object_set_data_full (GTK_OBJECT (window), "button1", button1,
		                          (GtkDestroyNotify) gtk_widget_unref);
/*		gtk_widget_show (button1);*/
		GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);
		gtk_widget_hide( button1 );

/*		gtk_widget_set_sensitive( button1, FALSE );*/
		gtk_object_set_data( GTK_OBJECT( button5 ), "reply", button1 );

		gtk_object_set_data( GTK_OBJECT( respond_text ), "button", button1 );
		gtk_object_set_data( GTK_OBJECT( button1 ), "menu", optionmenu_menu );
		gtk_object_set_data( GTK_OBJECT( button1 ), "contact", contact );
		gtk_object_set_data( GTK_OBJECT( button1 ), "text", respond_text );
		gtk_object_set_data( GTK_OBJECT( button1 ), "force", NULL );

		gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (window),
		                                        _("Read Next"), GNOME_STOCK_PIXMAP_FORWARD);
		button2 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
		gtk_widget_ref (button2);
		gtk_object_set_data_full (GTK_OBJECT (window), "button2", button2,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button2);
		GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

		((CONTACT_PTR)contact->data)->read_next = button2;
		if( g_slist_length( ((CONTACT_PTR)contact->data)->stored_messages ) == 1 )
			gtk_widget_set_sensitive( button2, FALSE );

		gtk_object_set_data( GTK_OBJECT( respond_text ), "readnext", button2 );
		gtk_object_set_data( GTK_OBJECT( button2 ), "contact", contact );
		gtk_object_set_data( GTK_OBJECT( button2 ), "text", text );
		gtk_object_set_data( GTK_OBJECT( button2 ), "time", timefield );

		gnome_dialog_append_button (GNOME_DIALOG (window), GNOME_STOCK_BUTTON_CLOSE);
		button3 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
		gtk_widget_ref (button3);
		gtk_object_set_data_full (GTK_OBJECT (window), "button3", button3,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button3);
		GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

		gtk_object_set_data( GTK_OBJECT( respond_text ), "cancel", button3 );

		gtk_object_set_data( GTK_OBJECT( button5 ), "readnext", button2 );
		gtk_object_set_data( GTK_OBJECT( button5 ), "cancel", button3 );

		if( toggles->full_reply )
		{
			gtk_widget_hide_all( hbox3 );
			gtk_widget_show( vbox2 );
			gtk_widget_show( button1 );
			gtk_widget_grab_focus( respond_text );
		}

		gtk_signal_connect_after( GTK_OBJECT( respond_text ), "key_press_event",
		                    GTK_SIGNAL_FUNC( ctrl_enter_cb ), NULL );

		gtk_signal_connect( GTK_OBJECT( button5 ), "key_press_event",
		                    GTK_SIGNAL_FUNC( ctrl_enter_cb ), NULL );

		gtk_signal_connect (GTK_OBJECT (button4), "clicked",
		                    GTK_SIGNAL_FUNC (grab_history),
		                    NULL);


		gtk_signal_connect (GTK_OBJECT (button1), "clicked",
		                    GTK_SIGNAL_FUNC (icq_sendmessage),
		                    NULL);

		gtk_signal_connect (GTK_OBJECT (button1), "clicked",
		                    GTK_SIGNAL_FUNC (icq_refresh_list),
		                    NULL);

		gtk_signal_connect_object (GTK_OBJECT (button1), "clicked",
		                           GTK_SIGNAL_FUNC (gtk_widget_destroy),
		                           GTK_OBJECT( window ));


		gtk_signal_connect (GTK_OBJECT (button2), "destroy",
		                    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
		                    &((CONTACT_PTR)contact->data)->read_next);

		gtk_signal_connect (GTK_OBJECT (button2), "clicked",
		                    GTK_SIGNAL_FUNC (read_next),
		                    NULL);


		gtk_signal_connect (GTK_OBJECT (button3), "clicked",
		                    GTK_SIGNAL_FUNC (icq_refresh_list),
		                    NULL);
		gtk_signal_connect_object (GTK_OBJECT (button3), "clicked",
		                           GTK_SIGNAL_FUNC (gtk_widget_destroy),
		                           GTK_OBJECT( window ));

		gtk_object_set_data( GTK_OBJECT( button5 ), "text", respond_text );

		if( toggles->full_reply )
			gtk_widget_grab_focus( respond_text );
		else
			gtk_widget_grab_focus (button5);

		gtk_widget_show( window );
	}

/*** User has received a CONTACT LIST ***/
	if( ((STORED_MESSAGE_PTR)message_text)->type == MESSAGE_CONT_LIST )
	{
		ptr = (char*)create_recd_contact_list_window();
		gtk_widget_show( GTK_WIDGET( ptr ) );

		contact_list_clist = gtk_object_get_data( GTK_OBJECT( ptr ), "clist" );

		ptr = ((STORED_MESSAGE_PTR)message_text)->message;
		ptr2 = ptr;

		ptr = strchr( ptr, '\xFE' );
		ptr[0] = 0;
		ptr2 = (ptr + 1);
		ptr = strchr( ptr2, '\xFE' );
		if( ptr == NULL )
			return;
		ptr[0] = 0;

		while( 1 )
		{
			ptr3 = (ptr + 1);
			ptr = strchr( ptr3, '\xFE' );
			if( ptr == NULL )
				break;
			ptr[0] = 0;

			buf[0] = ptr2;
			buf[1] = ptr3;
			gtk_clist_append( GTK_CLIST( contact_list_clist ), buf );

			ptr2 = (ptr + 1);
			ptr = strchr( ptr2, '\xFE' );
			if( ptr == NULL )
				break;
			ptr[0] = 0;
		}

		return;
	}

/*** User has added you to his/her list ***/
	if( ((STORED_MESSAGE_PTR)message_text)->type == MESSAGE_USER_ADD )
	{
		if (((CONTACT_PTR)contact->data)->info->sex == FEMALE)
		  sender = g_strdup_printf( _("%s has added you to her list"), nick );
		else if (((CONTACT_PTR)contact->data)->info->sex == MALE)
		  sender = g_strdup_printf( _("%s has added you to his list"), nick );
		else // No sex specified
		  sender = g_strdup_printf( _("%s has added you to his/her list"), nick );

		   
		gtk_widget_show( gnome_message_box_new( _( sender ),
		                                        GNOME_MESSAGE_BOX_INFO,
		                                        GNOME_STOCK_BUTTON_OK, NULL ) );
		g_free( sender );
		return;
	}

/*** Sender has authorized you to add him to your list ***/
	if( ((STORED_MESSAGE_PTR)message_text)->type == MESSAGE_USER_AUTHD )
	{
		sender = g_strdup_printf( _("You are authorized to add %s to your list"), nick );

		gtk_widget_show( gnome_message_box_new( _( sender ),
		                                        GNOME_MESSAGE_BOX_INFO,
		                                        GNOME_STOCK_BUTTON_OK, NULL ) );
		g_free( sender );
		return;
	}

/*** Sender has sent you a chat request ***/
	if( ((STORED_MESSAGE_PTR)message_text)->type == MESSAGE_CHAT_REQ )
	{
		window = gnome_dialog_new (_("GnomeICU: Chat Request"), NULL);
		gtk_object_set_data (GTK_OBJECT (window), "window", window);
		gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, FALSE);

		dialog_vbox1 = GNOME_DIALOG (window)->vbox;
		gtk_object_set_data (GTK_OBJECT (window), "dialog_vbox1", dialog_vbox1);
		gtk_widget_show (dialog_vbox1);

		vbox1 = gtk_vbox_new (FALSE, 0);
		gtk_widget_ref (vbox1);
		gtk_object_set_data_full (GTK_OBJECT (window), "vbox1", vbox1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (vbox1);
		gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

		sender = g_strdup_printf( _("Accept Chat Request from %s?\n"), nick );

		label1 = gtk_label_new ( sender );
		g_free( sender );
		gtk_widget_ref (label1);
		gtk_object_set_data_full (GTK_OBJECT (window), "label1", label1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label1);
		gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);

		label2 = gtk_label_new (_("Reason:"));
		gtk_widget_ref (label2);
		gtk_object_set_data_full (GTK_OBJECT (window), "label2", label2,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label2);
		gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 0);
		gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);
		gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

		scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
		gtk_widget_ref (scrolledwindow1);
		gtk_object_set_data_full (GTK_OBJECT (window), "scrolledwindow1", scrolledwindow1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (scrolledwindow1);
		gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

		text = gtk_text_new (NULL, NULL);
		gtk_widget_ref (text);
		gtk_object_set_data_full (GTK_OBJECT (window), "text", text,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (text);
		gtk_text_set_word_wrap( GTK_TEXT( text ), TRUE );
		gtk_text_set_editable( GTK_TEXT( text ), FALSE );
		if( ((CONTACT_PTR)contact->data)->chat_reason != NULL )
		{
				gtk_text_insert( GTK_TEXT( text ), NULL, NULL, NULL,
				                 ((CONTACT_PTR)contact->data)->chat_reason, -1 );
				g_free( ((CONTACT_PTR)contact->data)->chat_reason );
				((CONTACT_PTR)contact->data)->chat_reason = NULL;
		}
		else
				gtk_text_insert( GTK_TEXT( text ), NULL, NULL, NULL,
				                 _("No reason given"), -1 );

		gtk_container_add (GTK_CONTAINER (scrolledwindow1), text);

		dialog_action_area1 = GNOME_DIALOG (window)->action_area;
		gtk_object_set_data (GTK_OBJECT (window), "dialog_action_area1", dialog_action_area1);
		gtk_widget_show (dialog_action_area1);
		gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

		gnome_dialog_append_button (GNOME_DIALOG (window), GNOME_STOCK_BUTTON_YES );
		button1 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
		gtk_widget_ref (button1);
		gtk_object_set_data_full (GTK_OBJECT (window), "button1", button1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button1);
		GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

		gtk_signal_connect( GTK_OBJECT( button1 ), "clicked",
		                    GTK_SIGNAL_FUNC( icq_accept_chat ),
		                    contact );

		gtk_signal_connect_object( GTK_OBJECT( button1 ), "clicked",
		                           (GtkSignalFunc) gtk_widget_destroy,
		                           GTK_OBJECT( window ) );

		gnome_dialog_append_button (GNOME_DIALOG (window), GNOME_STOCK_BUTTON_NO);
		button3 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
		gtk_widget_ref (button3);
		gtk_object_set_data_full (GTK_OBJECT (window), "button3", button3,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button3);
		GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

		gtk_signal_connect( GTK_OBJECT( button3 ), "clicked",
		                    GTK_SIGNAL_FUNC( icq_refuse_chat ),
		                    contact );

		gtk_signal_connect_object( GTK_OBJECT( button3 ), "clicked",
		                           (GtkSignalFunc) gtk_widget_destroy,
		                           GTK_OBJECT( window ) );

		gtk_widget_show_all( window );
	}

/*** Sender has offered you a file ***/
	if( ((STORED_MESSAGE_PTR)message_text)->type == MESSAGE_FILE_REQ )
	{
		window = gnome_dialog_new (_("GnomeICU: File Offer"), NULL);
		gtk_object_set_data (GTK_OBJECT (window), "window", window);
		gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, FALSE);

		dialog_vbox1 = GNOME_DIALOG (window)->vbox;
		gtk_object_set_data (GTK_OBJECT (window), "dialog_vbox1", dialog_vbox1);
		gtk_widget_show (dialog_vbox1);

		vbox1 = gtk_vbox_new (FALSE, 4);
		gtk_widget_ref (vbox1);
		gtk_object_set_data_full (GTK_OBJECT (window), "vbox1", vbox1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (vbox1);
		gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

		sender = g_strdup_printf( _("Accept file from %s?\n"), nick );

		label1 = gtk_label_new ( sender );
		g_free( sender );
		gtk_widget_ref (label1);
		gtk_object_set_data_full (GTK_OBJECT (window), "label1", label1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label1);
		gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);

		table = gtk_table_new (4, 3, FALSE);
		gtk_widget_ref (table);
		gtk_object_set_data_full (GTK_OBJECT (window), "table", table,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (table);
		gtk_box_pack_start (GTK_BOX (vbox1), table, TRUE, TRUE, 0);

		label3 = gtk_label_new (_("Size:"));
		gtk_widget_ref (label3);
		gtk_object_set_data_full (GTK_OBJECT (window), "label3", label3,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label3);
		gtk_table_attach (GTK_TABLE (table), label3, 0, 1, 1, 2,
		                  (GtkAttachOptions) (GTK_FILL),
		                  (GtkAttachOptions) (0), 0, 0);
		gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_RIGHT);
		gtk_misc_set_alignment (GTK_MISC (label3), 1, 0.5);

		label2 = gtk_label_new (_("Name:"));
		gtk_widget_ref (label2);
		gtk_object_set_data_full (GTK_OBJECT (window), "label2", label2,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label2);
		gtk_table_attach (GTK_TABLE (table), label2, 0, 1, 0, 1,
		                  (GtkAttachOptions) (GTK_FILL),
		                  (GtkAttachOptions) (0), 0, 0);
		gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_RIGHT);
		gtk_misc_set_alignment (GTK_MISC (label2), 1, 0.5);

		label5 = gtk_label_new ("");
		gtk_widget_ref (label5);
		gtk_object_set_data_full (GTK_OBJECT (window), "label5", label5,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label5);
		gtk_table_attach (GTK_TABLE (table), label5, 1, 2, 2, 3,
		                  (GtkAttachOptions) (0),
		                  (GtkAttachOptions) (0), 0, 0);

		label4 = gtk_label_new (_("\nReason:"));
		gtk_widget_ref (label4);
		gtk_object_set_data_full (GTK_OBJECT (window), "label4", label4,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label4);
		gtk_table_attach (GTK_TABLE (table), label4, 0, 2, 2, 3,
		                  (GtkAttachOptions) (GTK_FILL),
		                  (GtkAttachOptions) (0), 0, 0);
		gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_LEFT);
		gtk_misc_set_alignment (GTK_MISC (label4), 7.45058e-09, 0.5);

		label6 = gtk_label_new ("");
		gtk_widget_ref (label6);
		gtk_object_set_data_full (GTK_OBJECT (window), "label6", label6,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label6);
		gtk_table_attach (GTK_TABLE (table), label6, 2, 3, 2, 3,
		                  (GtkAttachOptions) (0),
		                  (GtkAttachOptions) (0), 0, 0);

		scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
		gtk_widget_ref (scrolledwindow1);
		gtk_object_set_data_full (GTK_OBJECT (window), "scrolledwindow1", scrolledwindow1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (scrolledwindow1);
		gtk_table_attach (GTK_TABLE (table), scrolledwindow1, 0, 3, 3, 4,
		                  (GtkAttachOptions) (GTK_FILL),
		                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

		text = gtk_text_new (NULL, NULL);
		gtk_widget_ref (text);
		gtk_object_set_data_full (GTK_OBJECT (window), "text", text,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (text);
		gtk_text_set_word_wrap( GTK_TEXT( text ), TRUE );
		gtk_text_set_editable( GTK_TEXT( text ), FALSE );
		gtk_container_add (GTK_CONTAINER (scrolledwindow1), text);

		gtk_text_insert( GTK_TEXT( text ), NULL, NULL, NULL,
		                 ((STORED_MESSAGE_PTR)message_text)->message, -1 );

		entry1 = gtk_entry_new ();
		gtk_widget_ref (entry1);
		gtk_object_set_data_full (GTK_OBJECT (window), "entry1", entry1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (entry1);
		gtk_entry_set_editable( GTK_ENTRY( entry1 ), FALSE );
		sender = g_strdup_printf( "%d", ((CONTACT_PTR)contact->data)->file_size );
		gtk_entry_set_text( GTK_ENTRY( entry1 ), sender );
		g_free( sender );
		gtk_table_attach (GTK_TABLE (table), entry1, 1, 3, 1, 2,
		                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		                  (GtkAttachOptions) (0), 0, 0);

		entry2 = gtk_entry_new ();
		gtk_widget_ref (entry2);
		gtk_object_set_data_full (GTK_OBJECT (window), "entry2", entry2,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (entry2);
		gtk_entry_set_editable( GTK_ENTRY( entry2 ), FALSE );
		gtk_entry_set_text( GTK_ENTRY( entry2 ), ((CONTACT_PTR)contact->data)->file_short_name );
		gtk_table_attach (GTK_TABLE (table), entry2, 1, 3, 0, 1,
		                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		                  (GtkAttachOptions) (0), 0, 0);

		dialog_action_area1 = GNOME_DIALOG (window)->action_area;
		gtk_object_set_data (GTK_OBJECT (window), "dialog_action_area1", dialog_action_area1);
		gtk_widget_show (dialog_action_area1);
		gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

		gnome_dialog_append_button (GNOME_DIALOG (window), GNOME_STOCK_BUTTON_YES);
		button1 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
		gtk_widget_ref (button1);
		gtk_object_set_data_full (GTK_OBJECT (window), "button1", button1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button1);
		GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

		gnome_dialog_append_button (GNOME_DIALOG (window), GNOME_STOCK_BUTTON_NO);
		button3 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
		gtk_widget_ref (button3);
		gtk_object_set_data_full (GTK_OBJECT (window), "button3", button3,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button3);
		GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

		gtk_signal_connect (GTK_OBJECT (button1), "clicked",
		                    GTK_SIGNAL_FUNC (icq_accept_file),
		                    contact);
		gtk_signal_connect_object (GTK_OBJECT (button1), "clicked",
		                           GTK_SIGNAL_FUNC (gtk_widget_destroy),
		                           GTK_OBJECT( window ));
		gtk_signal_connect (GTK_OBJECT (button3), "clicked",
		                    GTK_SIGNAL_FUNC (icq_refuse_file),
		                    contact);
		gtk_signal_connect_object (GTK_OBJECT (button3), "clicked",
		                           GTK_SIGNAL_FUNC (gtk_widget_destroy),
		                           GTK_OBJECT( window ));

		gtk_widget_show_all( window );
	}

	if( ((STORED_MESSAGE_PTR)message_text)->type == MESSAGE_AUTH_REQ )
	{
		window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
		mainbox = gtk_vbox_new( FALSE, 0 );
		gtk_container_add( GTK_CONTAINER( window ), mainbox );
		gtk_widget_show( mainbox );

		message = gtk_label_new( sender );

		gtk_box_pack_start( GTK_BOX( mainbox ), message, FALSE, FALSE, 0 );
		gtk_widget_show( message );

		sender = g_strdup_printf( _("Auth Request from %s:"), nick );

		gtk_window_set_title( GTK_WINDOW( window ), _("GnomeICU: Authorization Request") );

		box = gtk_hbutton_box_new();
		gtk_button_box_set_layout(GTK_BUTTON_BOX(box), GTK_BUTTONBOX_END);
		allow_button = gtk_button_new_with_label( _("Allow") );
		gtk_container_add( GTK_CONTAINER( box ), allow_button );
		gtk_box_pack_start( GTK_BOX( mainbox ), box, FALSE, FALSE, 20 );
		gtk_object_set_data( GTK_OBJECT( allow_button ), "uin", GUINT_TO_POINTER(((CONTACT_PTR)contact->data)->uin) );
		gtk_signal_connect( GTK_OBJECT( allow_button ), "clicked",
		                    GTK_SIGNAL_FUNC( icq_allow_auth ), NULL );

		gtk_signal_connect_object(GTK_OBJECT( allow_button ), "clicked",
		                          (GtkSignalFunc) gtk_widget_destroy, 
	                             GTK_OBJECT( window ) );
		gtk_widget_show( allow_button );

		button = gtk_button_new_with_label( _("Refuse") );
		gtk_container_add(GTK_CONTAINER( box ), button);

		gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
		                          (GtkSignalFunc) gtk_widget_destroy, 
	                             GTK_OBJECT(window));
		gtk_widget_show( button );
		gtk_widget_show( box );

		gtk_widget_show_all( window );
	}

	if( ((STORED_MESSAGE_PTR)message_text)->type == MESSAGE_URL )
	{
		int url_len = 0, desc_len = 0;

		curl = g_strdup( ((STORED_MESSAGE_PTR)message_text)->message );
		cdesc = g_strdup( ((STORED_MESSAGE_PTR)message_text)->message );

		sender = g_strdup_printf( _("Received URL from %s:"), nick );

		for( cx = ( strchr( message_text->message, '\n' ) - message_text->message ); ; cx ++ )
		{
			if( ((STORED_MESSAGE_PTR)message_text)->message[ cx ] == '\xFE' ||
			    ((STORED_MESSAGE_PTR)message_text)->message[ cx ] == 0x00 )
			{
				if( ((STORED_MESSAGE_PTR)message_text)->message[ cx ] == 0x00 )
				{
					url_len = cx - 1 - desc_len - ( strchr( message_text->message, '\n' ) - message_text->message );
					break;
				}
				else
					desc_len = cx - 1 - ( strchr( message_text->message, '\n' ) - message_text->message );
			}
		}

		memcpy( cdesc, strchr( message_text->message, '\n' ) + 1, desc_len );
		cdesc[ desc_len ] = 0x00;
		memcpy( curl, strchr( message_text->message, '\n' ) + 2 + desc_len, url_len );
		curl[ url_len ]  = 0x00;
		
		window = gnome_dialog_new (_("GnomeICU: URL"), NULL);
		gtk_widget_set_name (window, "window");
		gtk_object_set_data (GTK_OBJECT (window), "window", window);
		gtk_widget_set_usize (window, 400, -2);
		GTK_WINDOW (window)->type = GTK_WINDOW_DIALOG;
		gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
		gtk_window_set_default_size( GTK_WINDOW( window ), 500, 150 );
		gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);
		gtk_window_set_wmclass (GTK_WINDOW (window), "window", "GnomeICU");
		gnome_dialog_set_close (GNOME_DIALOG (window), TRUE);

		dialog_vbox1 = GNOME_DIALOG (window)->vbox;
		gtk_widget_set_name (dialog_vbox1, "dialog_vbox1");
		gtk_object_set_data (GTK_OBJECT (window), "dialog_vbox1", dialog_vbox1);
		gtk_widget_show (dialog_vbox1);

		vbox1 = gtk_vbox_new (FALSE, 4);
		gtk_widget_set_name (vbox1, "vbox1");
		gtk_widget_ref (vbox1);
		gtk_object_set_data_full (GTK_OBJECT (window), "vbox1", vbox1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (vbox1);
		gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

		label1 = gtk_label_new ( sender );
		gtk_widget_set_name (label1, "label1");
		gtk_widget_ref (label1);
		gtk_object_set_data_full (GTK_OBJECT (window), "label1", label1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label1);
		gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
		gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
		gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

		table = gtk_table_new (2, 2, FALSE);
		gtk_widget_set_name (table, "table");
		gtk_widget_ref (table);
		gtk_object_set_data_full (GTK_OBJECT (window), "table", table,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (table);
		gtk_box_pack_start (GTK_BOX (vbox1), table, TRUE, TRUE, 0);
		gtk_table_set_row_spacings (GTK_TABLE (table), 4);
		gtk_table_set_col_spacings (GTK_TABLE (table), 4);

		hbox1 = gtk_hbox_new (FALSE, 4);
		gtk_widget_set_name (hbox1, "hbox1");
		gtk_widget_ref (hbox1);
		gtk_object_set_data_full (GTK_OBJECT (window), "hbox1", hbox1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (hbox1);
		gtk_table_attach (GTK_TABLE (table), hbox1, 1, 2, 0, 1,
		                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

		url = gtk_entry_new ();
		gtk_widget_set_name (url, "url");
		gtk_widget_ref (url);
		gtk_object_set_data_full (GTK_OBJECT (window), "url", url,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (url);
		gtk_box_pack_start (GTK_BOX (hbox1), url, TRUE, TRUE, 0);

		gtk_entry_set_text( GTK_ENTRY( url ), curl );

		button1 = gtk_button_new_with_label (_("Go To"));
		gtk_widget_set_name (button1, "button1");
		gtk_widget_ref (button1);
		gtk_object_set_data_full (GTK_OBJECT (window), "button1", button1,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button1);
		gtk_box_pack_start (GTK_BOX (hbox1), button1, FALSE, FALSE, 0);

		gtk_signal_connect( GTK_OBJECT( button1 ), "clicked",
		                    GTK_SIGNAL_FUNC( display_url ),
		                    g_strdup( curl ) );

		desc = gtk_entry_new ();
		gtk_widget_set_name (desc, "desc");
		gtk_widget_ref (desc);
		gtk_object_set_data_full (GTK_OBJECT (window), "desc", desc,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (desc);
		gtk_table_attach (GTK_TABLE (table), desc, 1, 2, 1, 2,
		                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		                  (GtkAttachOptions) (0), 0, 0);

		gtk_entry_set_text( GTK_ENTRY( desc ), cdesc );

		label2 = gtk_label_new (_("URL:"));
		gtk_widget_set_name (label2, "label2");
		gtk_widget_ref (label2);
		gtk_object_set_data_full (GTK_OBJECT (window), "label2", label2,
		                         (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label2);
		gtk_table_attach (GTK_TABLE (table), label2, 0, 1, 0, 1,
		                 (GtkAttachOptions) (GTK_FILL),
		                 (GtkAttachOptions) (GTK_FILL), 0, 0);
		gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_RIGHT);
		gtk_misc_set_alignment (GTK_MISC (label2), 1, 0.5);

		label3 = gtk_label_new (_("Description:"));
		gtk_widget_set_name (label3, "label3");
		gtk_widget_ref (label3);
		gtk_object_set_data_full (GTK_OBJECT (window), "label3", label3,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (label3);
		gtk_table_attach (GTK_TABLE (table), label3, 0, 1, 1, 2,
		                  (GtkAttachOptions) (GTK_FILL),
		                  (GtkAttachOptions) (GTK_FILL), 0, 0);
		gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_RIGHT);
		gtk_misc_set_alignment (GTK_MISC (label3), 1, 0.5);

		dialog_action_area1 = GNOME_DIALOG (window)->action_area;
		gtk_widget_set_name (dialog_action_area1, "dialog_action_area1");
		gtk_object_set_data (GTK_OBJECT (window), "dialog_action_area1", dialog_action_area1);
		gtk_widget_show (dialog_action_area1);
		gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
		gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

		gnome_dialog_append_button (GNOME_DIALOG (window), GNOME_STOCK_BUTTON_CLOSE);
		button2 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
		gtk_widget_set_name (button2, "button2");
		gtk_widget_ref (button2);
		gtk_object_set_data_full (GTK_OBJECT (window), "button2", button2,
		                          (GtkDestroyNotify) gtk_widget_unref);
		gtk_widget_show (button2);
		GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

		gtk_signal_connect_object( GTK_OBJECT( button2 ), "clicked",
		                           GTK_SIGNAL_FUNC( gtk_widget_destroy ),
		                           GTK_OBJECT( window ) );

		gtk_widget_show_all( window );

		g_free( curl );
		g_free( cdesc );
		g_free( sender );
	}
}

GtkWidget *create_menu ( void )
{ 
	GtkWidget *item;
	GtkWidget *menu;

#ifdef TRACE_FUNCTION
	g_print( "create_menu\n" );
#endif

	menu = gtk_menu_new();

	item = gtk_menu_item_new_with_label ( _("Online") );

	gtk_menu_append( GTK_MENU ( menu ), item );
	gtk_widget_show( item );

	gtk_signal_connect(GTK_OBJECT( item ), "activate",
	                   GTK_SIGNAL_FUNC(icq_set_status_online), NULL );

	item = gtk_menu_item_new_with_label ( _("Offline") );

	gtk_menu_append( GTK_MENU ( menu ), item );
	gtk_widget_show( item );

	gtk_signal_connect(GTK_OBJECT( item ), "activate",
	                   GTK_SIGNAL_FUNC(icq_set_status_offline), NULL );

	item = gtk_menu_item_new_with_label ( _("Away") );

	gtk_menu_append( GTK_MENU ( menu ), item );
	gtk_widget_show( item );

	gtk_signal_connect(GTK_OBJECT( item ), "activate",
	                   GTK_SIGNAL_FUNC(icq_set_status_away), NULL );

	item = gtk_menu_item_new_with_label ( _("Not Available") );

	gtk_menu_append( GTK_MENU ( menu ), item );
	gtk_widget_show( item );

	gtk_signal_connect(GTK_OBJECT( item ), "activate",
	                   GTK_SIGNAL_FUNC(icq_set_status_na), NULL );

	item = gtk_menu_item_new_with_label ( _("Free for Chat") );

	gtk_menu_append( GTK_MENU ( menu ), item );
	gtk_widget_show( item );

	gtk_signal_connect(GTK_OBJECT( item ), "activate",
	                   GTK_SIGNAL_FUNC(icq_set_status_ffc), NULL );

	item = gtk_menu_item_new_with_label ( _("Occupied") );

	gtk_menu_append( GTK_MENU ( menu ), item );
	gtk_widget_show( item );

	gtk_signal_connect(GTK_OBJECT( item ), "activate",
	                   GTK_SIGNAL_FUNC(icq_set_status_occ), NULL );

	item = gtk_menu_item_new_with_label ( _("Do Not Disturb") );

	gtk_menu_append( GTK_MENU ( menu ), item );
	gtk_widget_show( item );

	gtk_signal_connect(GTK_OBJECT( item ), "activate",
	                   GTK_SIGNAL_FUNC(icq_set_status_dnd), NULL );

	item = gtk_menu_item_new_with_label ( _("Invisible") );

	gtk_menu_append( GTK_MENU ( menu ), item );
	gtk_widget_show( item );

	gtk_signal_connect(GTK_OBJECT( item ), "activate",
	                   GTK_SIGNAL_FUNC(icq_set_status_invisible), NULL );

	return menu;
}

void show_contact_message( GSList *contact )
{
	if( ((CONTACT_PTR)contact->data)->read_next &&
	    GTK_IS_WIDGET( ((CONTACT_PTR)contact->data)->read_next ) &&
	    GTK_WIDGET_VISIBLE( ((CONTACT_PTR)contact->data)->read_next ) )
		gtk_signal_emit_by_name( GTK_OBJECT( ((CONTACT_PTR)contact->data)->read_next ), "clicked", MainData );

	if( g_slist_length( ((CONTACT_PTR)contact->data)->stored_messages ) )
	{
		icq_msgbox( ((STORED_MESSAGE_PTR)((CONTACT_PTR)contact->data)->stored_messages->data), ((CONTACT_PTR)contact->data)->uin );

		g_free( ((STORED_MESSAGE_PTR)((CONTACT_PTR)contact->data)->stored_messages->data)->message );
		g_free( ((CONTACT_PTR)contact->data)->stored_messages->data );
		((CONTACT_PTR)contact->data)->stored_messages = g_slist_remove( ((CONTACT_PTR)contact->data)->stored_messages, ((CONTACT_PTR)contact->data)->stored_messages->data );

		if( g_slist_length( ((CONTACT_PTR)contact->data)->stored_messages ) )
		{
			((CONTACT_PTR)contact->data)->icon_p = GetIcon_p( ((CONTACT_PTR)contact->data)->status );
			((CONTACT_PTR)contact->data)->icon_b = GetIcon_b( ((CONTACT_PTR)contact->data)->status );
			((CONTACT_PTR)contact->data)->need_update = 1;
		}

		Show_Quick_Status();
		applet_update( Current_Status, NULL );
	}
}
	
void icq_sendmessage( GtkWidget *button, gpointer data )
{
	gchar *buf;
	GtkWidget *entry;
	gpointer forced;

	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "icq_sendmessage\n" );
#endif

	if( gtk_object_get_data( GTK_OBJECT( button ), "menu" ) != NULL )
		contact = gtk_object_get_data( GTK_OBJECT( gtk_menu_get_active( gtk_object_get_data( GTK_OBJECT( button ), "menu" ) ) ), "contact" );
	else
		contact = gtk_object_get_data( GTK_OBJECT( button ), "contact" );
	entry = gtk_object_get_data( GTK_OBJECT( button ), "text" );
	forced = gtk_object_get_data( GTK_OBJECT( button ), "force" );

	buf = gtk_editable_get_chars( GTK_EDITABLE( entry ), 0, -1 );

	icq_sendmsg( ((CONTACT_PTR)contact->data)->uin, buf, ( forced == NULL ? 0 : 1 ) );

	g_free( buf );
}

gint icq_sendmessage_window( GtkWidget *clist, GdkEventButton *event, gpointer data )
{
	int row, column;

	GtkWidget *dialog1;
	GtkWidget *dialog_vbox1;
	GtkWidget *vbox1;
	GtkWidget *hbox1;
	GtkWidget *label1;
	GtkWidget *send_message_optionmenu;
	GtkWidget *send_message_optionmenu_menu;
	GtkWidget *menuitem;
	GtkWidget *scrolledwindow1;
	GtkWidget *text1;
	GtkWidget *dialog_action_area1;
	GtkWidget *button1;
	GtkWidget *button3;

	int cx, picked_user;
	gchar *nick = NULL;
	int uin;

	int key_select = FALSE;

	GSList *contact, *contact2;

#ifdef TRACE_FUNCTION
	g_print( "icq_sendmessage_window\n" );
#endif

	if( event == NULL )
		return FALSE;

	if( event->type == GDK_KEY_PRESS )
	{
 		if( ((GdkEventKey*)event)->keyval != GDK_Return && ((GdkEventKey*)event)->keyval != ' ' )
			return FALSE;
		key_select = TRUE;
	}

	gtk_clist_get_selection_info( GTK_CLIST( clist ), event->x, event->y,
	                              &row, &column );

	if( event->type == GDK_KEY_PRESS )
		row = GTK_CLIST(clist)->focus_row;

	if( !GTK_IS_SCROLLBAR( clist ) && key_select == FALSE &&
	    event->button == 3 )
		 gtk_clist_select_row( GTK_CLIST( clist ), row, column );

	if( key_select == FALSE )
	{
		if(  event->type != GDK_2BUTTON_PRESS && event->button == 1 )
			return FALSE;

		if( event->type != GDK_BUTTON_PRESS && event->button == 3 )
			return FALSE;

		if( event->button == 2 )
			return FALSE;
	}

	uin = GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), row ) );

	if( uin == 0 )
		return FALSE;

	contact = Contacts;

	while( contact != NULL )
	{
		if( ((CONTACT_PTR)contact->data)->uin == uin )
			break;
		contact = contact->next;
	}

	if( data != NULL )
		contact = (GSList*)data;

	if( contact == NULL )
		return FALSE;

	contact2 = contact;

	nick = ((CONTACT_PTR)contact->data)->nick;

	if( event->button == 3 ||
	    ( event->type == GDK_KEY_PRESS && ((GdkEventKey*)event)->keyval == ' ' ) )
	{
		GtkWidget *personal_menu;
		GdkEventButton *bevent = (GdkEventButton *) event;

		GtkWidget *pm, *item_box, *item_label;
		GtkWidget *item;
		personal_menu = gtk_menu_new();

		item_box = gtk_hbox_new( FALSE, 0 );
		pm = gtk_pixmap_new( icon_message_pixmap, icon_message_bitmap );
		gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
		gtk_widget_show( pm );
		item = gtk_menu_item_new();
		item_label = gtk_label_new( _("  Message...") );
		gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
		gtk_widget_show( item_label );
		gtk_container_add( GTK_CONTAINER( item ), item_box );
		gtk_widget_show( item_box );
		gtk_menu_append( GTK_MENU( personal_menu ), item );
		gtk_signal_connect( GTK_OBJECT( item ), "activate",
		                    GTK_SIGNAL_FUNC( icq_sendmessage_window_from_menu ),
		                    NULL );
		gtk_widget_show( item );

		item_box = gtk_hbox_new( FALSE, 0 );
		pm = gtk_pixmap_new( icon_url_pixmap, icon_url_bitmap );
		gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
		gtk_widget_show( pm );
		item = gtk_menu_item_new();
		item_label = gtk_label_new( _("  URL...") );
		gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
		gtk_widget_show( item_label );
		gtk_container_add( GTK_CONTAINER( item ), item_box );
		gtk_widget_show( item_box );
		gtk_menu_append( GTK_MENU( personal_menu ), item );
		gtk_signal_connect( GTK_OBJECT( item ), "activate",
		                    GTK_SIGNAL_FUNC( send_url_window_default ),
		                    NULL );
		gtk_widget_show( item );

		if( ((CONTACT_PTR)contact->data)->status != STATUS_OFFLINE )
		{
			item_box = gtk_hbox_new( FALSE, 0 );
			pm = gtk_pixmap_new( icon_chat_pixmap, icon_chat_bitmap );
			gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
			gtk_widget_show( pm );
			item = gtk_menu_item_new();
			item_label = gtk_label_new( _("  Chat Request...") );
			gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
			gtk_widget_show( item_label );
			gtk_container_add( GTK_CONTAINER( item ), item_box );
			gtk_widget_show( item_box );
			gtk_menu_append( GTK_MENU( personal_menu ), item );
			gtk_signal_connect( GTK_OBJECT( item ), "activate",
			                    GTK_SIGNAL_FUNC( request_chat ),
			                    NULL );
			gtk_widget_show( item );

			item_box = gtk_hbox_new( FALSE, 0 );
			pm = gtk_pixmap_new( icon_file_pixmap, icon_file_bitmap );
			gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
			gtk_widget_show( pm );
			item = gtk_menu_item_new();
			item_label = gtk_label_new( _("  File Request...") );
			gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
			gtk_widget_show( item_label );
			gtk_container_add( GTK_CONTAINER( item ), item_box );
			gtk_widget_show( item_box );
			gtk_menu_append( GTK_MENU( personal_menu ), item );
			gtk_signal_connect( GTK_OBJECT( item ), "activate",
			                    GTK_SIGNAL_FUNC( request_file_default ),
			                    NULL );
			gtk_widget_show( item );

			item_box = gtk_hbox_new( FALSE, 0 );
			pm = gtk_pixmap_new( icon_contact_pixmap, icon_contact_bitmap );
			gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
			gtk_widget_show( pm );
			item = gtk_menu_item_new();
			item_label = gtk_label_new( _("  Contact List...") );
			gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
			gtk_widget_show( item_label );
			gtk_container_add( GTK_CONTAINER( item ), item_box );
			gtk_widget_show( item_box );
			gtk_menu_append( GTK_MENU( personal_menu ), item );
			gtk_signal_connect( GTK_OBJECT( item ), "activate",
			                    GTK_SIGNAL_FUNC( get_contact_list ),
			                    NULL );
			gtk_widget_show( item );
		}

		item = gtk_menu_item_new();
		gtk_widget_set_sensitive( item, FALSE );
		gtk_menu_append( GTK_MENU( personal_menu ), item );
		gtk_widget_show( item );

		item_box = gtk_hbox_new( FALSE, 0 );
		pm = gtk_pixmap_new( icon_info_pixmap, icon_info_bitmap );
		gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
		gtk_widget_show( pm );
		item = gtk_menu_item_new();
		item_label = gtk_label_new( _("  User Info...") );
		gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
		gtk_widget_show( item_label );
		gtk_container_add( GTK_CONTAINER( item ), item_box );
		gtk_widget_show( item_box );
		gtk_menu_append( GTK_MENU( personal_menu ), item );
		gtk_signal_connect( GTK_OBJECT( item ), "activate",
		                    GTK_SIGNAL_FUNC( show_personal_info ), NULL );
		gtk_widget_show( item );

		if( (((CONTACT_PTR)contact->data)->status & 0xffff) != STATUS_ONLINE &&
		    ((CONTACT_PTR)contact->data)->status != STATUS_OFFLINE &&
		    (((CONTACT_PTR)contact->data)->status & 0xffff) !=  STATUS_INVISIBLE &&
		    (((CONTACT_PTR)contact->data)->status & 0xffff) != STATUS_FREE_CHAT )
		{
			item_box = gtk_hbox_new( FALSE, 0 );
			pm = gtk_pixmap_new( GetIcon_p( ((CONTACT_PTR)contact->data)->status ),
			                     GetIcon_b( ((CONTACT_PTR)contact->data)->status ) );
			gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
			gtk_widget_show( pm );
			item = gtk_menu_item_new();
			switch( ((CONTACT_PTR)contact->data)->status & 0xffff )
			{
				case STATUS_AWAY:
					item_label = gtk_label_new( _("  Read Away Message") );
					break;
				case STATUS_NA:
					item_label = gtk_label_new( _("  Read N/A Message") );
					break;
				case STATUS_OCCUPIED:
					item_label = gtk_label_new( _("  Read Occupied Message") );
					break;
				case STATUS_DND:
					item_label = gtk_label_new( _("  Read Do not Disturb Message") );
					break;
				default:
					item_label = gtk_label_new( _("  Read Away Message") );
					break;
			}

			gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
			gtk_widget_show( item_label );
			gtk_container_add( GTK_CONTAINER( item ), item_box );
			gtk_widget_show( item_box );
			gtk_menu_append( GTK_MENU( personal_menu ), item );
			gtk_signal_connect( GTK_OBJECT( item ), "activate",
			                    GTK_SIGNAL_FUNC( retrieve_away_message ), NULL );
			gtk_widget_show( item );
		}

		if( ((CONTACT_PTR)contact->data)->status != STATUS_OFFLINE )
		{
			item_box = gtk_hbox_new( FALSE, 0 );
			pm = gtk_pixmap_new( icon_info_pixmap, icon_info_bitmap );
			gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
			gtk_widget_show( pm );
			item = gtk_menu_item_new();
			item_label = gtk_label_new( _("  Version...") );
			gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
			gtk_widget_show( item_label );
			gtk_container_add( GTK_CONTAINER( item ), item_box );
			gtk_widget_show( item_box );
			gtk_menu_append( GTK_MENU( personal_menu ), item );
			gtk_signal_connect( GTK_OBJECT( item ), "activate",
			                    GTK_SIGNAL_FUNC( retrieve_version ), NULL );
			gtk_widget_show( item );
		}

		item_box = gtk_hbox_new( FALSE, 0 );
		pm = gtk_pixmap_new( icon_hist_pixmap, icon_hist_bitmap );
		gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
		gtk_widget_show( pm );
		item = gtk_menu_item_new();
		item_label = gtk_label_new( _("  History...") );
		gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
		gtk_widget_show( item_label );
		gtk_container_add( GTK_CONTAINER( item ), item_box );
		gtk_widget_show( item_box );
		gtk_menu_append( GTK_MENU( personal_menu ), item );
		gtk_signal_connect( GTK_OBJECT( item ), "activate",
		                    GTK_SIGNAL_FUNC( display_mess_history ), NULL );
		gtk_widget_show( item );

		item = gtk_menu_item_new();
		gtk_widget_set_sensitive( item, FALSE );
		gtk_menu_append( GTK_MENU( personal_menu ), item );
		gtk_widget_show( item );

		if( ((CONTACT_PTR)contact->data)->inlist == FALSE )
		{
			item_box = gtk_hbox_new( FALSE, 0 );
			pm = gtk_pixmap_new( icon_auth_pixmap, icon_auth_bitmap );
			gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
			gtk_widget_show( pm );
			item = gtk_menu_item_new();
			item_label = gtk_label_new( _("  Add To List") );
			gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
			gtk_widget_show( item_label );
			gtk_container_add( GTK_CONTAINER( item ), item_box );
			gtk_widget_show( item_box );
			gtk_menu_append( GTK_MENU( personal_menu ), item );
			gtk_signal_connect( GTK_OBJECT( item ), "activate",
			                    GTK_SIGNAL_FUNC( add_user_to_list ), NULL );
			gtk_widget_show( item );

			item = gtk_menu_item_new();
			gtk_widget_set_sensitive( item, FALSE );
			gtk_menu_append( GTK_MENU( personal_menu ), item );
			gtk_widget_show( item );
		}

		item_box = gtk_hbox_new( FALSE, 0 );
		pm = gtk_pixmap_new( icon_auth_pixmap, icon_auth_bitmap );
		gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
		gtk_widget_show( pm );
		item = gtk_menu_item_new();
		item_label = gtk_label_new( _("  Authorize User") );
		gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
		gtk_widget_show( item_label );
		gtk_container_add( GTK_CONTAINER( item ), item_box );
		gtk_widget_show( item_box );
		gtk_menu_append( GTK_MENU( personal_menu ), item );
		gtk_signal_connect( GTK_OBJECT( item ), "activate",
		                    GTK_SIGNAL_FUNC( authorize_uin ), NULL );
		gtk_widget_show( item );

		item_box = gtk_hbox_new( FALSE, 0 );
		pm = gtk_pixmap_new( icon_rename_pixmap, icon_rename_bitmap );
		gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
		gtk_widget_show( pm );
		item = gtk_menu_item_new();
		item_label = gtk_label_new( _("  Rename User...") );
		gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
		gtk_widget_show( item_label );
		gtk_container_add( GTK_CONTAINER( item ), item_box );
		gtk_widget_show( item_box );
		gtk_menu_append( GTK_MENU( personal_menu ), item );
		gtk_signal_connect( GTK_OBJECT( item ), "activate",
		                    GTK_SIGNAL_FUNC( change_nick_window ), NULL );
		gtk_widget_show( item );

		item_box = gtk_hbox_new( FALSE, 0 );
		pm = gtk_pixmap_new( icon_cancel_pixmap, icon_cancel_bitmap );
		gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
		gtk_widget_show( pm );
		item = gtk_menu_item_new();
		item_label = gtk_label_new( _("  Remove User...") );
		gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
		gtk_widget_show( item_label );
		gtk_container_add( GTK_CONTAINER( item ), item_box );
		gtk_widget_show( item_box );
		gtk_menu_append( GTK_MENU( personal_menu ), item );
		gtk_signal_connect( GTK_OBJECT( item ), "activate",
		                    GTK_SIGNAL_FUNC( remove_user_question ), NULL );
		gtk_widget_show( item );

		gtk_menu_popup( GTK_MENU( personal_menu ), NULL, NULL, NULL, NULL,
		                bevent->button, bevent->time );
		return FALSE;
	}

	if( g_slist_length( ((CONTACT_PTR)contact->data)->stored_messages ) )
	{
		show_contact_message( contact );
		return FALSE;
	}

	dialog1 = gnome_dialog_new (_("GnomeICU: Send Message"), NULL);
	gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
	gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);

	dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
	gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
	gtk_widget_show (dialog_vbox1);

	vbox1 = gtk_vbox_new (FALSE, 4);
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox1", vbox1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

	hbox1 = gtk_hbox_new (FALSE, 4);
	gtk_widget_ref (hbox1);
	gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox1", hbox1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

	label1 = gtk_label_new (_("Send message to:"));
	gtk_widget_ref (label1);
	gtk_object_set_data_full (GTK_OBJECT (dialog1), "label1", label1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label1);
	gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (label1), 1, 0.5);

	send_message_optionmenu = gtk_option_menu_new ();
	gtk_widget_ref (send_message_optionmenu);
	gtk_object_set_data_full (GTK_OBJECT (dialog1), "send_message_optionmenu", send_message_optionmenu,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (send_message_optionmenu);
	gtk_box_pack_start (GTK_BOX (hbox1), send_message_optionmenu, FALSE, FALSE, 0);
	send_message_optionmenu_menu = gtk_menu_new ();

	contact2 = Contacts;
	cx = 0;
	picked_user = 0;
	while( contact2 != NULL )
	{
		menuitem = gtk_menu_item_new_with_label( ((CONTACT_PTR)contact2->data)->nick );
		gtk_widget_show (menuitem);
		gtk_object_set_data( GTK_OBJECT( menuitem ), "contact", contact2 );
		gtk_menu_append (GTK_MENU (send_message_optionmenu_menu), menuitem);
		if( contact == contact2 )
			picked_user = cx;
		contact2 = contact2->next;
		cx++;
	}

	gtk_option_menu_set_menu( GTK_OPTION_MENU( send_message_optionmenu ), send_message_optionmenu_menu );
	gtk_option_menu_set_history( GTK_OPTION_MENU( send_message_optionmenu ), picked_user );

	button1 = gtk_button_new_with_label( _("History") );
	gtk_box_pack_end( GTK_BOX( hbox1 ), button1, FALSE, FALSE, 0 );
	gtk_object_set_data( GTK_OBJECT( button1 ), "menu", send_message_optionmenu_menu );
	gtk_signal_connect( GTK_OBJECT( button1 ), "clicked",
	                    GTK_SIGNAL_FUNC( grab_history ), NULL );
	gtk_widget_show( button1 );

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (scrolledwindow1);
	gtk_object_set_data_full (GTK_OBJECT (dialog1), "scrolledwindow1", scrolledwindow1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

	text1 = gtk_text_new (NULL, NULL);
	gtk_widget_ref (text1);
	gtk_object_set_data_full (GTK_OBJECT (dialog1), "text1", text1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_text_set_editable( GTK_TEXT( text1 ), TRUE );
	gtk_text_set_word_wrap( GTK_TEXT( text1 ), TRUE );

	gtk_object_set_data( GTK_OBJECT( text1 ), "nick", ((CONTACT_PTR)contact->data)->nick );
	gtk_object_set_data( GTK_OBJECT( text1 ), "force", NULL );

	gtk_signal_connect_after( GTK_OBJECT( text1 ), "key_press_event",
	                    GTK_SIGNAL_FUNC( ctrl_enter_cb ), NULL );

	gtk_widget_show (text1);
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), text1);
	gtk_widget_set_usize (text1, 375, 146);
	GTK_WIDGET_SET_FLAGS (text1, GTK_CAN_DEFAULT);

	dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
	gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
	gtk_widget_show (dialog_action_area1);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (dialog1),
	                                        _("Send"), GNOME_STOCK_PIXMAP_JUMP_TO);
	button1 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
	gtk_widget_ref (button1);
	gtk_object_set_data_full (GTK_OBJECT (dialog1), "button1", button1,
	                          (GtkDestroyNotify) gtk_widget_unref);

	gtk_object_set_data( GTK_OBJECT( text1 ), "button", button1 );
	gtk_object_set_data( GTK_OBJECT( button1 ), "text", text1 );
	gtk_object_set_data( GTK_OBJECT( button1 ), "force", NULL );
	gtk_object_set_data( GTK_OBJECT( button1 ), "contact", contact );
	gtk_object_set_data( GTK_OBJECT( button1 ), "menu", send_message_optionmenu_menu );

	gtk_signal_connect( GTK_OBJECT( button1 ), "clicked",
	                    GTK_SIGNAL_FUNC( icq_sendmessage ), NULL );
	gtk_signal_connect_object( GTK_OBJECT( button1 ), "clicked",
	                           (GtkSignalFunc) gtk_widget_destroy,
	                           GTK_OBJECT( dialog1 ) );

	gtk_widget_show (button1);
	GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

	gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CLOSE);
	button3 = g_list_last (GNOME_DIALOG (dialog1)->buttons)->data;
	gtk_widget_ref (button3);
	gtk_object_set_data_full (GTK_OBJECT (dialog1), "button3", button3,
	                          (GtkDestroyNotify) gtk_widget_unref);

	gtk_object_set_data( GTK_OBJECT( text1 ), "cancel", button3 );

	gtk_signal_connect_object( GTK_OBJECT( button3 ), "clicked",
	                           (GtkSignalFunc) gtk_widget_destroy,
	                           GTK_OBJECT( dialog1 ) );

	gtk_widget_show (button3);
	GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

	gtk_widget_show_all( dialog1 );

	gtk_widget_grab_focus( text1 );
	gtk_widget_grab_default (text1);

	return FALSE;
}

void icq_sendmessage_window_from_menu( GtkWidget *widget, gpointer data )
{
	GdkEventButton *event = g_new0( GdkEventButton, 1 );
	GSList *contact = Contacts;

#ifdef TRACE_FUNCTION
	g_print( "icq_sendmessage_window_from_menu\n" );
#endif

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( MainData->lb_userwin ), GPOINTER_TO_INT( GTK_CLIST( MainData->lb_userwin )->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	event->type = GDK_KEY_PRESS;
	((GdkEventKey*)event)->keyval = GDK_Return;

	icq_sendmessage_window( MainData->lb_userwin, event, contact );

	g_free( event );
}

int stay_connected( void )
{
#ifdef TRACE_FUNCTION
	g_print( "stay_connected\n" );
#endif

	Keep_Alive();
	return TRUE;
}

void show_personal_info( GtkWidget *widget, gpointer data )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );

	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "show_personal_info\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	((CONTACT_PTR)contact->data)->info_was_requested = TRUE;

	info_req_99( ((CONTACT_PTR)contact->data)->uin );

	dump_personal_info( ((CONTACT_PTR)contact->data)->uin );
}

void dump_personal_info( UIN_T uin )
{
	gchar *sts, *string = NULL;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *button;
	GtkWidget *vbutton;
	GtkWidget *notebook;
	GtkWidget *mainbox;
	GtkWidget *hbox;
	GtkWidget *scrollbar;

	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "dump_personal_info\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( uin == ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	/* hvrfixme: where is current_ip and port freed up again?? */
	if( ((CONTACT_PTR)contact->data)->current_ip != -1 )
	{
		((CONTACT_PTR)contact->data)->info->ip = g_strdup_printf( "%u.%u.%u.%u",
		         (BYTE) (((CONTACT_PTR)contact->data)->current_ip >> 24),
		         (BYTE) (((CONTACT_PTR)contact->data)->current_ip >> 16),
		         (BYTE) (((CONTACT_PTR)contact->data)->current_ip >> 8),
		         (BYTE) (((CONTACT_PTR)contact->data)->current_ip) );
		((CONTACT_PTR)contact->data)->info->port = g_strdup_printf( "%u", ((CONTACT_PTR)contact->data)->port );
	}
	else
	{
		((CONTACT_PTR)contact->data)->info->ip = g_strdup( _("N/A") );
		((CONTACT_PTR)contact->data)->info->port = g_strdup( _("N/A") );
	}

	if( ((CONTACT_PTR)contact->data)->info->status != NULL )
		g_free( ((CONTACT_PTR)contact->data)->info->status );

	if( ((CONTACT_PTR)contact->data)->status == STATUS_OFFLINE )
		sts = "Offline";
	else
		switch( ((CONTACT_PTR)contact->data)->status & 0xffff )
		{
			case STATUS_ONLINE:
				sts = "Online"; break;
			case STATUS_AWAY:
				sts = "Away"; break;
			case STATUS_NA:
				sts = "N/A"; break;
			case STATUS_FREE_CHAT:
				sts = "Free for Chat"; break;
			case STATUS_OCCUPIED:
				sts = "Occupied"; break;
			case STATUS_DND:
				sts = "Do not Disturb"; break;
			case STATUS_INVISIBLE:
				sts = "Invisible"; break;
			default:
				sts = "Unknown"; break;
		}

	((CONTACT_PTR)contact->data)->info->status = g_strdup_printf( "%08X (%s)",
	                                             ((CONTACT_PTR)contact->data)->status,
	                                             sts );

	if( ((CONTACT_PTR)contact->data)->info->window == NULL )
	{
		if( ((CONTACT_PTR)contact->data)->info_was_requested == FALSE )
			return;
		((CONTACT_PTR)contact->data)->info_was_requested = FALSE;
		((CONTACT_PTR)contact->data)->info->window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
		gtk_container_border_width( GTK_CONTAINER( ((CONTACT_PTR)contact->data)->info->window ), 10 );
		gtk_window_set_title( GTK_WINDOW( ((CONTACT_PTR)contact->data)->info->window ), _("GnomeICU: User Information") );

		gtk_signal_connect( GTK_OBJECT( ((CONTACT_PTR)contact->data)->info->window ), "destroy",
		                    (GtkSignalFunc)clear_i_window,
		                    GINT_TO_POINTER( ((CONTACT_PTR)contact->data)->uin ) );

		mainbox = gtk_vbox_new( FALSE, 0 );
		gtk_container_add( GTK_CONTAINER( ((CONTACT_PTR)contact->data)->info->window ), mainbox );

		notebook = gtk_notebook_new();
		gtk_notebook_set_tab_pos( GTK_NOTEBOOK( notebook ), GTK_POS_TOP );
		gtk_box_pack_start( GTK_BOX( mainbox ), notebook, FALSE, FALSE, 10 );

/** General Tab **/
		table = gtk_table_new( 4, 7, FALSE );
		gtk_container_set_border_width( GTK_CONTAINER( table ), 7 );
		gtk_notebook_append_page( GTK_NOTEBOOK( notebook ), table,
		                          gtk_label_new( _("General") ) );

		label = gtk_label_new( _("UIN:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Nick:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 3, 4, 0, 1, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );
		
		label = gtk_label_new( _("F. Name:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("L. Name:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 3, 4, 2, 3, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );
		
		label = gtk_label_new( _("Sex:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 3, 4, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Age:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 3, 4, 3, 4, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Alias:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 3, 4, 1, 2, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Auth:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		((CONTACT_PTR)contact->data)->info->w_uin = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_uin,
		                  1, 2, 0, 1, GTK_FILL, GTK_FILL, 5, 5 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_uin ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_uin );

		((CONTACT_PTR)contact->data)->info->w_nick = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_nick,
		                  4, 7, 0, 1, GTK_FILL, GTK_FILL, 5, 5 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_nick ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_nick );

		((CONTACT_PTR)contact->data)->info->w_first = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_first,
		                  1, 3, 2, 3, GTK_FILL, GTK_FILL, 5, 5 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_first ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_first );

		((CONTACT_PTR)contact->data)->info->w_last = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_last,
		                  4, 7, 2, 3, GTK_FILL, GTK_FILL, 5, 5 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_last ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_last );

		((CONTACT_PTR)contact->data)->info->w_sex = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_sex,
		                  1, 2, 3, 4, GTK_FILL, GTK_FILL, 5, 5 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_sex ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_sex );

		((CONTACT_PTR)contact->data)->info->w_age = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_age,
		                  4, 7, 3, 4, GTK_FILL, GTK_FILL, 5, 5 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_age ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_age );

		((CONTACT_PTR)contact->data)->info->w_alias = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_alias,
		                  4, 6, 1, 2, GTK_FILL, GTK_FILL, 5, 5 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_alias ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_alias );

		button = gtk_button_new_with_label( _("Set") );
		gtk_signal_connect( GTK_OBJECT( button ), "clicked",
		                    (GtkSignalFunc)change_nick_window,
		                    GINT_TO_POINTER( ((CONTACT_PTR)contact->data)->uin ) );
		gtk_table_attach( GTK_TABLE( table ), button,
		                  6, 7, 1, 2, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 5 );
		gtk_widget_show( button );

		((CONTACT_PTR)contact->data)->info->w_auth = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_auth,
		                  1, 2, 1, 2, GTK_FILL, GTK_FILL, 5, 5 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_auth ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_auth );

		gtk_widget_show( table );

/** Internet Tab **/
		table = gtk_table_new( 7, 2, FALSE );
		gtk_container_set_border_width( GTK_CONTAINER( table ), 7 );
		gtk_notebook_append_page( GTK_NOTEBOOK( notebook ), table,
		                          gtk_label_new( _("Internet") ) );

		label = gtk_label_new( _("IP:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Port:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Status:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_hseparator_new();
		gtk_table_attach( GTK_TABLE( table ), label, 0, 2, 3, 4, GTK_FILL, 0, 0, 7 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Email:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 4, 5, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Other Email:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 5, 6, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Old Email:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 6, 7, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		((CONTACT_PTR)contact->data)->info->w_ip = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_ip,
		                  1, 2, 0, 1, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_ip ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_ip );

		((CONTACT_PTR)contact->data)->info->w_port = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_port,
		                  1, 2, 1, 2, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_port ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_port );

		((CONTACT_PTR)contact->data)->info->w_status = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_status,
		                  1, 2, 2, 3, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_status ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_status );

		((CONTACT_PTR)contact->data)->info->w_email = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_email,
		                  1, 2, 4, 5, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_widget_set_usize( ((CONTACT_PTR)contact->data)->info->w_email, 200, 0 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_email ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_email );

		((CONTACT_PTR)contact->data)->info->w_email2 = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_email2,
		                  1, 2, 5, 6, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_widget_set_usize( ((CONTACT_PTR)contact->data)->info->w_email2, 200, 0 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_email2 ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_email2 );

		((CONTACT_PTR)contact->data)->info->w_email3 = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_email3,
		                  1, 2, 6, 7, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_widget_set_usize( ((CONTACT_PTR)contact->data)->info->w_email3, 200, 0 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_email3 ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_email3 );

		gtk_widget_show( table );

/** Work Tab **/
		table = gtk_table_new( 8, 4, FALSE );
		gtk_container_set_border_width( GTK_CONTAINER( table ), 7 );
		gtk_notebook_append_page( GTK_NOTEBOOK( notebook ), table,
		                          gtk_label_new( _("Work") ) );

		label = gtk_label_new( _("Job Position:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Department:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 2, 3, 0, 1, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_hseparator_new();
		gtk_table_attach( GTK_TABLE( table ), label, 0, 4, 1, 2, GTK_FILL, 0, 0, 7 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Company:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Address:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 3, 4, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("City:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 4, 5, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("State:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 2, 3, 4, 5, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_hseparator_new();
		gtk_table_attach( GTK_TABLE( table ), label, 0, 4, 5, 6, GTK_FILL, 0, 0, 7 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Phone:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 6, 7, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Fax:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 2, 3, 6, 7, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Homepage:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 7, 8, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		((CONTACT_PTR)contact->data)->info->w_job_pos = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_job_pos,
		                  1, 2, 0, 1, GTK_FILL, GTK_FILL, 5, 7 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_job_pos ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_job_pos );

		((CONTACT_PTR)contact->data)->info->w_department = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_department,
		                  3, 4, 0, 1, GTK_FILL, GTK_FILL, 5, 7 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_department ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_department );

		((CONTACT_PTR)contact->data)->info->w_company_name = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_company_name,
		                  1, 2, 2, 3, GTK_FILL, GTK_FILL, 5, 3 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_company_name ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_company_name );

		((CONTACT_PTR)contact->data)->info->w_work_address = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_work_address,
		                  1, 4, 3, 4, GTK_FILL, GTK_FILL, 5, 0 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_address ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_work_address );

		((CONTACT_PTR)contact->data)->info->w_work_city = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_work_city,
		                  1, 2, 4, 5, GTK_FILL, GTK_FILL, 5, 3 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_city ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_work_city );

		((CONTACT_PTR)contact->data)->info->w_work_state = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_work_state,
		                  3, 4, 4, 5, GTK_FILL, GTK_FILL, 5, 3 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_state ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_work_state );

		((CONTACT_PTR)contact->data)->info->w_work_phone = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_work_phone,
		                  1, 2, 6, 7, GTK_FILL, GTK_FILL, 5, 7 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_phone ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_work_phone );

		((CONTACT_PTR)contact->data)->info->w_work_fax = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_work_fax,
		                  3, 4, 6, 7, GTK_FILL, GTK_FILL, 5, 7 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_fax ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_work_fax );

		((CONTACT_PTR)contact->data)->info->w_work_homepage = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_work_homepage,
		                  1, 4, 7, 8, GTK_FILL, GTK_FILL, 5, 3 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_homepage ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_work_homepage );

		gtk_widget_show( table );

/** Location Tab **/
		table = gtk_table_new( 6, 4, FALSE );
		gtk_container_set_border_width( GTK_CONTAINER( table ), 7 );
		gtk_notebook_append_page( GTK_NOTEBOOK( notebook ), table,
		                          gtk_label_new( _("Location") ) );

		label = gtk_label_new( _("Address:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("City:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("State:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 2, 3, 1, 2, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Zip:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Country:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 2, 3, 2, 3, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_hseparator_new();
		gtk_table_attach( GTK_TABLE( table ), label, 0, 4, 3, 4, GTK_FILL, 0, 0, 7 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Phone:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 4, 5, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Fax:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 2, 3, 4, 5, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("Cellular:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 5, 6, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		((CONTACT_PTR)contact->data)->info->w_street = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_street,
		                  1, 4, 0, 1, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_street ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_street );

		((CONTACT_PTR)contact->data)->info->w_city = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_city,
		                  1, 2, 1, 2, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_city ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_city );

		((CONTACT_PTR)contact->data)->info->w_state = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_state,
		                  3, 4, 1, 2, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_state ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_state );

		((CONTACT_PTR)contact->data)->info->w_zip = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_zip,
		                  1, 2, 2, 3, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_zip ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_zip );

		((CONTACT_PTR)contact->data)->info->w_country = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_country,
		                  3, 4, 2, 3, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_country ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_country );

		((CONTACT_PTR)contact->data)->info->w_phone = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_phone,
		                  1, 2, 4, 5, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_phone ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_phone );

		((CONTACT_PTR)contact->data)->info->w_fax = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_fax,
		                  3, 4, 4, 5, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_fax ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_fax );

		((CONTACT_PTR)contact->data)->info->w_cellular = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_cellular,
		                  1, 2, 5, 6, GTK_FILL, GTK_FILL, 5, 2 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_cellular ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_cellular );

		gtk_widget_show( table );

/** About Tab **/
		table = gtk_table_new( 3, 3, FALSE );
		gtk_container_set_border_width( GTK_CONTAINER( table ), 7 );
		gtk_notebook_append_page( GTK_NOTEBOOK( notebook ), table,
		                          gtk_label_new( _("About") ) );

		label = gtk_label_new( _("Home Page:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 1, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		label = gtk_label_new( _("About:") );
		gtk_misc_set_alignment( GTK_MISC( label ), 0, 1 );
		gtk_table_attach( GTK_TABLE( table ), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0 );
		gtk_widget_show( label );

		((CONTACT_PTR)contact->data)->info->w_homepage = gtk_entry_new();

		gtk_table_attach( GTK_TABLE( table ), ((CONTACT_PTR)contact->data)->info->w_homepage,
		                  1, 2, 0, 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 5, 5 );
		gtk_entry_set_editable( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_homepage ), FALSE );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_homepage );

		vbutton = gtk_button_new_with_label( _("View") );
		gtk_table_attach( GTK_TABLE( table ), vbutton,
		                  2, 3, 0, 1, GTK_FILL, 0, 0, 0 );

		gtk_signal_connect( GTK_OBJECT( vbutton ), "clicked",
		                    GTK_SIGNAL_FUNC( display_url_from_entry ),
		                    ((CONTACT_PTR)contact->data)->info->w_homepage );

		gtk_widget_show( vbutton );

		hbox = gtk_hbox_new( FALSE, 0 );
		((CONTACT_PTR)contact->data)->info->w_about = gtk_text_new( NULL, NULL );
		gtk_text_set_word_wrap( GTK_TEXT( ((CONTACT_PTR)contact->data)->info->w_about ), TRUE );

		gtk_box_pack_start( GTK_BOX( hbox ), ((CONTACT_PTR)contact->data)->info->w_about, TRUE, TRUE, 0 );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->w_about );

		scrollbar = gtk_vscrollbar_new( GTK_TEXT( ((CONTACT_PTR)contact->data)->info->w_about )->vadj );
		gtk_box_pack_start( GTK_BOX( hbox ), scrollbar, FALSE, FALSE, 0 );
		gtk_widget_show( scrollbar );


		gtk_table_attach( GTK_TABLE( table ), hbox,
		                  0, 3, 2, 3, GTK_FILL | GTK_EXPAND,
		                  GTK_FILL | GTK_EXPAND, 5, 5 );

		gtk_widget_show( hbox );
		gtk_widget_show( table );

		hbox = gtk_hbutton_box_new();
		gtk_button_box_set_layout( GTK_BUTTON_BOX( hbox ),
		                           GTK_BUTTONBOX_END );

		button = gtk_button_new_with_label( _("Close") );
		gtk_box_pack_end( GTK_BOX( mainbox ), hbox, FALSE, FALSE, 0 );
		gtk_box_pack_end( GTK_BOX( hbox ), button, FALSE, FALSE, 0 );
		gtk_signal_connect_object( GTK_OBJECT( button ), "clicked",
		                    (GtkSignalFunc) gtk_widget_destroy,
		                    GTK_OBJECT( ((CONTACT_PTR)contact->data)->info->window ) );
		gtk_widget_show( hbox );
		gtk_widget_show( button );

		gtk_widget_show( notebook );
		gtk_widget_show( mainbox );
		gtk_widget_show( ((CONTACT_PTR)contact->data)->info->window );
	}

	gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_uin ),
	                    g_strdup_printf( "%u", ((CONTACT_PTR)contact->data)->uin ) );

	if( ((CONTACT_PTR)contact->data)->info->nick != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_nick ), ((CONTACT_PTR)contact->data)->info->nick );

	if( ((CONTACT_PTR)contact->data)->info->first != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_first ), ((CONTACT_PTR)contact->data)->info->first );

	if( ((CONTACT_PTR)contact->data)->info->last != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_last ), ((CONTACT_PTR)contact->data)->info->last );

	if( ((CONTACT_PTR)contact->data)->info->sex == MALE )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_sex ), _("Male") );
	else if( ((CONTACT_PTR)contact->data)->info->sex == FEMALE )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_sex ), _("Female") );
	else
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_sex ), _("Unknown") );

	gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_age ), 
			    (string = ((CONTACT_PTR)contact->data)->info->age 
			     ? g_strdup_printf( "%d", ((CONTACT_PTR)contact->data)->info->age % 1000 ) 
			     : g_strdup( _("Unknown") )));
	g_free( string );

	if( ((CONTACT_PTR)contact->data)->nick != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_alias ), ((CONTACT_PTR)contact->data)->nick );

	if( ((CONTACT_PTR)contact->data)->info->uin == 0 )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_auth ), _("Unknown") );

	if( ((CONTACT_PTR)contact->data)->info->ip != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_ip ), ((CONTACT_PTR)contact->data)->info->ip );

	if( ((CONTACT_PTR)contact->data)->info->port != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_port ), ((CONTACT_PTR)contact->data)->info->port );

	if( ((CONTACT_PTR)contact->data)->info->status != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_status ), ((CONTACT_PTR)contact->data)->info->status );

	if( ((CONTACT_PTR)contact->data)->info->email != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_email ), ((CONTACT_PTR)contact->data)->info->email );

	if( ((CONTACT_PTR)contact->data)->info->email2 != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_email2 ), ((CONTACT_PTR)contact->data)->info->email2 );

	if( ((CONTACT_PTR)contact->data)->info->email3 != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_email3 ), ((CONTACT_PTR)contact->data)->info->email3 );

	if( ((CONTACT_PTR)contact->data)->info->phone != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_phone ), ((CONTACT_PTR)contact->data)->info->phone );

	if( ((CONTACT_PTR)contact->data)->info->city != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_city ), ((CONTACT_PTR)contact->data)->info->city );

	if( ((CONTACT_PTR)contact->data)->info->state != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_state ), ((CONTACT_PTR)contact->data)->info->state );

	if( ((CONTACT_PTR)contact->data)->info->homepage != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_homepage ), ((CONTACT_PTR)contact->data)->info->homepage );

	if( ((CONTACT_PTR)contact->data)->info->about != NULL )
	{
		gtk_text_freeze( GTK_TEXT( ((CONTACT_PTR)contact->data)->info->w_about ) );
		gtk_text_insert( GTK_TEXT( ((CONTACT_PTR)contact->data)->info->w_about ), NULL, NULL, NULL,
		                 ((CONTACT_PTR)contact->data)->info->about, -1 );
		gtk_text_thaw( GTK_TEXT( ((CONTACT_PTR)contact->data)->info->w_about ) );
	}

	if( ((CONTACT_PTR)contact->data)->info->job_pos != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_job_pos ), ((CONTACT_PTR)contact->data)->info->job_pos );

	if( ((CONTACT_PTR)contact->data)->info->department != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_department ), ((CONTACT_PTR)contact->data)->info->department );

	if( ((CONTACT_PTR)contact->data)->info->company_name != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_company_name ), ((CONTACT_PTR)contact->data)->info->company_name );

	if( ((CONTACT_PTR)contact->data)->info->work_address != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_address ), ((CONTACT_PTR)contact->data)->info->work_address );

	if( ((CONTACT_PTR)contact->data)->info->work_city != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_city ), ((CONTACT_PTR)contact->data)->info->work_city );

	if( ((CONTACT_PTR)contact->data)->info->work_state != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_state ), ((CONTACT_PTR)contact->data)->info->work_state );

	if( ((CONTACT_PTR)contact->data)->info->work_phone != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_phone ), ((CONTACT_PTR)contact->data)->info->work_phone );

	if( ((CONTACT_PTR)contact->data)->info->work_fax != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_fax ), ((CONTACT_PTR)contact->data)->info->work_fax );

	if( ((CONTACT_PTR)contact->data)->info->work_homepage != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_work_homepage ), ((CONTACT_PTR)contact->data)->info->work_homepage );

	if( ((CONTACT_PTR)contact->data)->info->street != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_street ), ((CONTACT_PTR)contact->data)->info->street );

	if( ((CONTACT_PTR)contact->data)->info->city != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_city ), ((CONTACT_PTR)contact->data)->info->city );

	if( ((CONTACT_PTR)contact->data)->info->state != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_state ), ((CONTACT_PTR)contact->data)->info->state );

	if( ((CONTACT_PTR)contact->data)->info->zip != 0 )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_zip ),
		                    g_strdup_printf( "%d", ((CONTACT_PTR)contact->data)->info->zip ) );

	if( ((CONTACT_PTR)contact->data)->info->country != 0 )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_country ),
		                    Get_Country_Name( ((CONTACT_PTR)contact->data)->info->country ) );

	if( ((CONTACT_PTR)contact->data)->info->phone != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_phone ), ((CONTACT_PTR)contact->data)->info->phone );

	if( ((CONTACT_PTR)contact->data)->info->fax != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_fax ), ((CONTACT_PTR)contact->data)->info->fax );

	if( ((CONTACT_PTR)contact->data)->info->cellular != NULL )
		gtk_entry_set_text( GTK_ENTRY( ((CONTACT_PTR)contact->data)->info->w_cellular ), ((CONTACT_PTR)contact->data)->info->cellular );
}

void authorize_uin( GtkWidget *widget, gpointer data )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );
	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "authorize_uin\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;
	
	icq_sendauthmsg( ((CONTACT_PTR)contact->data)->uin );
}

void add_user_to_list( GtkWidget *widget, gpointer data )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );
	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "authorize_uin\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;
	
	((CONTACT_PTR)contact->data)->inlist = TRUE;
	Show_Quick_Status_lower( UPDATE_NOTLIST |
	                         ( ((CONTACT_PTR)contact->data)->status == STATUS_OFFLINE ? UPDATE_OFFLINE : UPDATE_ONLINE ),
	                         NULL );
}

void search_window( GtkWidget *widget, gpointer data )
{
#ifdef TRACE_FUNCTION
	g_print( "search_window\n" );
#endif

	gtk_widget_show( create_AddContact() );
}

void display_mess_history( GtkWidget *widget, gpointer data )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );
	GtkWidget *MessageHistory;
	GtkWidget *dialog_vbox7;
	GtkWidget *scrolledwindow8;
	GtkWidget *text;
	GtkWidget *dialog_action_area7;
	GtkWidget *button23;

	gchar *filename;
	FILE *file;

	GSList *contact;
	GList *listed_history = NULL, *rev_list;

	char *buffer;
	char *ptr;
	struct stat fs;
	int nchars;
	int cy = 0;

#ifdef TRACE_FUNCTION
	g_print( "display_mess_history\n" );
#endif

	if( data == NULL )
	{
		contact = Contacts;

		while( contact != NULL )
		{
			if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
			    ((CONTACT_PTR)contact->data)->uin )
				break;
			contact = contact->next;
		}
	}
	else
		contact = data;

	if( contact == NULL )
		return;

	filename = g_strdup_printf( "%s/.icq/history/%u", getenv( "HOME" ),
				    ((CONTACT_PTR)contact->data)->uin );		 

	file = fopen( filename, "r" );

	if( file == NULL )
	{
		OK_Box( _("Error:"), _("No message history available") );
		g_free( filename );
		return;
	}

	MessageHistory = gnome_dialog_new ( g_strdup_printf( _("GnomeICU: Message History: %s"), ((CONTACT_PTR)contact->data)->nick ), NULL);
	gtk_widget_set_name (MessageHistory, "MessageHistory");
	gtk_object_set_data (GTK_OBJECT (MessageHistory), "MessageHistory", MessageHistory);
	gtk_window_set_default_size ( GTK_WINDOW( MessageHistory ), 400, 300);
	GTK_WINDOW (MessageHistory)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_position (GTK_WINDOW (MessageHistory), GTK_WIN_POS_CENTER);
	gtk_window_set_policy (GTK_WINDOW (MessageHistory), TRUE, TRUE, FALSE);
	gtk_window_set_wmclass (GTK_WINDOW (MessageHistory), "MessageHistory", "GnomeICU");

	dialog_vbox7 = GNOME_DIALOG (MessageHistory)->vbox;
	gtk_widget_set_name (dialog_vbox7, "dialog_vbox7");
	gtk_object_set_data (GTK_OBJECT (MessageHistory), "dialog_vbox7", dialog_vbox7);
	gtk_widget_show (dialog_vbox7);

	scrolledwindow8 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_set_name (scrolledwindow8, "scrolledwindow8");
	gtk_widget_ref (scrolledwindow8);
	gtk_object_set_data_full (GTK_OBJECT (MessageHistory), "scrolledwindow8", scrolledwindow8,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow8);
	gtk_box_pack_start (GTK_BOX (dialog_vbox7), scrolledwindow8, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow8), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

	text = gtk_text_new (NULL, NULL);
	gtk_widget_set_name (text, "text");
	gtk_widget_ref (text);
	gtk_object_set_data_full (GTK_OBJECT (MessageHistory), "text", text,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_set_usize( text, 225, 200 );
	gtk_widget_show (text);
	gtk_container_add (GTK_CONTAINER (scrolledwindow8), text);

	dialog_action_area7 = GNOME_DIALOG (MessageHistory)->action_area;
	gtk_widget_set_name (dialog_action_area7, "dialog_action_area7");
	gtk_object_set_data (GTK_OBJECT (MessageHistory), "dialog_action_area7", dialog_action_area7);
	gtk_widget_show (dialog_action_area7);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area7), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area7), 8);

	gnome_dialog_append_button (GNOME_DIALOG (MessageHistory), GNOME_STOCK_BUTTON_CLOSE);
	button23 = g_list_last (GNOME_DIALOG (MessageHistory)->buttons)->data;
	gtk_widget_set_name (button23, "button23");
	gtk_widget_ref (button23);
	gtk_object_set_data_full (GTK_OBJECT (MessageHistory), "button23", button23,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_signal_connect_object( GTK_OBJECT( button23 ), "clicked",
	                           GTK_SIGNAL_FUNC( gtk_widget_destroy ),
	                           GTK_OBJECT( MessageHistory ) );
	gtk_widget_show (button23);
	GTK_WIDGET_SET_FLAGS (button23, GTK_CAN_DEFAULT);

	gtk_widget_grab_default (button23);


	stat( filename, &fs );
	buffer = (char*)g_malloc( fs.st_size + 16 );
	nchars = fread( buffer, 1, fs.st_size, file );
	fclose( file );

	ptr = buffer;

	while( cy < nchars )
	{
		if( buffer[ cy ] == '*' )
		{
			if( buffer[ cy + 1 ] == '*' && buffer[ cy + 2 ] == '*' &&
			    buffer[ cy + 3 ] == ' ' )
			{
				buffer[ cy ] = 0x00;
				listed_history = g_list_append( listed_history, g_strdup( ptr ) );

				ptr = &buffer[ cy ];
				buffer[ cy ] = '*';

				while( gtk_events_pending() )
					gtk_main_iteration();
			}
		}

		cy ++;
	}

	if( strlen( ptr ) )
	{
		buffer[ cy++ ] = '\n';
		buffer[ cy ] = 0x00;
		listed_history = g_list_append( listed_history, g_strdup( ptr ) );
	}

	g_free( buffer );

	g_free(filename);

	rev_list = g_list_last( listed_history );

	while( rev_list->prev != NULL )
	{
		gtk_text_insert( GTK_TEXT( text ), NULL, NULL, NULL,
		                 rev_list->data, -1 );
		g_free( rev_list->data );
		rev_list = rev_list->prev;
	}

	gtk_widget_show_all( MessageHistory );

	g_list_free( listed_history );
}

void read_next( GtkWidget *widget, gpointer data )
{
	GtkWidget *text;
	GtkWidget *timefield;
	gchar buf[100];
	gchar *msgtime;
	gchar *new_str;

	int cx;

	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "read_next\n" );
#endif

	contact = gtk_object_get_data( GTK_OBJECT( widget ), "contact" );
	text = gtk_object_get_data( GTK_OBJECT( widget ), "text" );
	timefield = gtk_object_get_data( GTK_OBJECT( widget ), "time" );

	if( contact == NULL )
		return;

	if( g_slist_length( ((CONTACT_PTR)contact->data)->stored_messages ) )
	{
		sprintf( buf, _("Message from %s:"), ((CONTACT_PTR)contact->data)->nick );
		gtk_text_set_point( GTK_TEXT( text ), 0 );
		gtk_text_forward_delete( GTK_TEXT( text ),
		                         gtk_text_get_length( GTK_TEXT( text ) ) );

		msgtime = g_strdup( ((STORED_MESSAGE_PTR)((CONTACT_PTR)contact->data)->stored_messages->data)->message );
		cx = 0;
		while(msgtime[cx++] != '\n' );
		msgtime[cx] = 0;
		gtk_entry_set_text( GTK_ENTRY( timefield ), _(msgtime) );
		g_free( msgtime );

/* Strip \r's out of text - \r's are for Windows people */
		new_str = stripr( ((STORED_MESSAGE_PTR)((CONTACT_PTR)contact->data)->stored_messages->data)->message );
		g_free( ((STORED_MESSAGE_PTR)((CONTACT_PTR)contact->data)->stored_messages->data)->message );
		((STORED_MESSAGE_PTR)((CONTACT_PTR)contact->data)->stored_messages->data)->message = new_str;

		gtk_text_insert( GTK_TEXT( text ), NULL, NULL, NULL,
		                 ( cx + ((STORED_MESSAGE_PTR)((CONTACT_PTR)contact->data)->stored_messages->data)->message ), -1 );

		if( g_slist_length( ((CONTACT_PTR)contact->data)->stored_messages ) <= 1 ||
		    ( g_slist_length( ((CONTACT_PTR)contact->data)->stored_messages ) &&
		    ((STORED_MESSAGE_PTR)((CONTACT_PTR)contact->data)->stored_messages->data)->type != MESSAGE_TEXT ) )
			gtk_widget_set_sensitive( ((CONTACT_PTR)contact->data)->read_next, FALSE );

		g_free( ((STORED_MESSAGE_PTR)((CONTACT_PTR)contact->data)->stored_messages->data)->message );
		g_free( ((CONTACT_PTR)contact->data)->stored_messages->data );
		((CONTACT_PTR)contact->data)->stored_messages = g_slist_remove( ((CONTACT_PTR)contact->data)->stored_messages, ((CONTACT_PTR)contact->data)->stored_messages->data );

		applet_update( Current_Status, NULL );
		if( g_slist_length( ((CONTACT_PTR)contact->data)->stored_messages ) == 0 )
		{
			((CONTACT_PTR)contact->data)->icon_p = GetIcon_p( ((CONTACT_PTR)contact->data)->status );
			((CONTACT_PTR)contact->data)->icon_b = GetIcon_b( ((CONTACT_PTR)contact->data)->status );
			((CONTACT_PTR)contact->data)->need_update = 1;
		}
		Show_Quick_Status();
	}

	return;
}

void send_chat_request( GtkWidget *widget, GtkWidget *text )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );

	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "send_chat_request\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	TCPSendChatRequest( ((CONTACT_PTR)contact->data)->uin,
	                    gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 ) );
}

void send_file_request( GtkWidget *widget, gpointer data )
{
	gchar *filename;
	gchar *msg;
	GtkText *text;
	GtkEntry *entry;
	UIN_T uin;

	uin = GPOINTER_TO_INT( gtk_object_get_data( GTK_OBJECT( widget ), "uin" ) );
	text = gtk_object_get_data( GTK_OBJECT( widget ), "reason" );
	entry = gtk_object_get_data( GTK_OBJECT( widget ), "file" );

	filename = g_strdup( gtk_entry_get_text( entry ) );
	msg = g_strdup( gtk_editable_get_chars( GTK_EDITABLE( text ), 0, -1 ) );

	TCPSendFileRequest( uin, msg, filename );

	g_free( msg );
	g_free( filename );
}

void request_chat( GtkWidget *widget, gpointer data )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );

	GtkWidget *window;
	GtkWidget *dialog_vbox2;
	GtkWidget *vbox1;
	GtkWidget *label9;
	GtkWidget *scrolledwindow2;
	GtkWidget *text2;
	GtkWidget *dialog_action_area2;
	GtkWidget *button6;
	GtkWidget *button7;

	GSList *contact;

	gchar *str;

#ifdef TRACE_FUNCTION
	g_print( "request_chat\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	window = gnome_dialog_new (_("GnomeICU: Chat Request"), NULL);
	gtk_object_set_data (GTK_OBJECT (window), "window", window);
	gtk_window_set_default_size (GTK_WINDOW (window), 250, 210);
	gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);

	dialog_vbox2 = GNOME_DIALOG (window)->vbox;
	gtk_object_set_data (GTK_OBJECT (window), "dialog_vbox2", dialog_vbox2);
	gtk_widget_show (dialog_vbox2);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (window), "vbox1", vbox1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox2), vbox1, TRUE, TRUE, 0);

	str = g_strdup_printf( _("Send Chat Request to %s:\n\nReason:"),
	                       ((CONTACT_PTR)contact->data)->nick );

	label9 = gtk_label_new ( str );
	g_free( str );
	gtk_widget_ref (label9);
	gtk_object_set_data_full (GTK_OBJECT (window), "label9", label9,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label9);
	gtk_box_pack_start (GTK_BOX (vbox1), label9, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label9), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label9), 7.45058e-09, 0.5);

	scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (scrolledwindow2);
	gtk_object_set_data_full (GTK_OBJECT (window), "scrolledwindow2", scrolledwindow2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow2);
	gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow2, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

	text2 = gtk_text_new (NULL, NULL);
	gtk_widget_ref (text2);
	gtk_object_set_data_full (GTK_OBJECT (window), "text2", text2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (text2);
	GTK_WIDGET_SET_FLAGS (text2, GTK_CAN_FOCUS);
	gtk_text_set_word_wrap( GTK_TEXT( text2 ), TRUE );
	gtk_text_set_editable( GTK_TEXT( text2 ), TRUE );
	gtk_container_add (GTK_CONTAINER (scrolledwindow2), text2);

	dialog_action_area2 = GNOME_DIALOG (window)->action_area;
	gtk_object_set_data (GTK_OBJECT (window), "dialog_action_area2", dialog_action_area2);
	gtk_widget_show (dialog_action_area2);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area2), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area2), 8);

	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (window),
	                                        _("Send"), GNOME_STOCK_PIXMAP_JUMP_TO);
	button6 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
	gtk_widget_ref (button6);
	gtk_object_set_data_full (GTK_OBJECT (window), "button6", button6,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (button6);
	GTK_WIDGET_SET_FLAGS (button6, GTK_CAN_DEFAULT);

	gnome_dialog_append_button (GNOME_DIALOG (window), GNOME_STOCK_BUTTON_CLOSE);
	button7 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
	gtk_widget_ref (button7);
	gtk_object_set_data_full (GTK_OBJECT (window), "button7", button7,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (button7);
	GTK_WIDGET_SET_FLAGS (button7, GTK_CAN_DEFAULT);

	gtk_object_set_data( GTK_OBJECT( text2 ), "button", button6 );
	gtk_object_set_data( GTK_OBJECT( text2 ), "cancel", button7 );

	gtk_signal_connect_after (GTK_OBJECT (text2), "key_press_event",
	                    GTK_SIGNAL_FUNC (ctrl_enter_cb),
	                    NULL);

	gtk_signal_connect (GTK_OBJECT (button6), "clicked",
	                    GTK_SIGNAL_FUNC (send_chat_request),
	                    text2);

	gtk_signal_connect_object (GTK_OBJECT (button6), "clicked",
	                           GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                           GTK_OBJECT( window ));

	gtk_signal_connect_object (GTK_OBJECT (button7), "clicked",
	                           GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                           GTK_OBJECT( window ));

	gtk_widget_grab_focus( text2 );
	gtk_widget_show_all( window );
}

void request_file( GtkWidget *widget, char *defaultfile )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );

	GSList *contact;

	GtkWidget *window;
	GtkWidget *dialog_vbox1;
	GtkWidget *vbox1;
	GtkWidget *label3;
	GtkWidget *hbox1;
	GtkWidget *label1;
	GtkWidget *fileentry1;
	GtkWidget *combo_entry1;
	GtkWidget *label2;
	GtkWidget *scrolledwindow1;
	GtkWidget *text1;
	GtkWidget *dialog_action_area1;
	GtkWidget *button1;
	GtkWidget *button2;

	gchar *str;

#ifdef TRACE_FUNCTION
	g_print( "request_file\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	window = gnome_dialog_new (_("GnomeICU: Send File"), NULL);
	gtk_object_set_data (GTK_OBJECT (window), "window", window);
	gtk_window_set_default_size( GTK_WINDOW( window ), 350, 250 );
	gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);

	dialog_vbox1 = GNOME_DIALOG (window)->vbox;
	gtk_object_set_data (GTK_OBJECT (window), "dialog_vbox1", dialog_vbox1);
	gtk_widget_show (dialog_vbox1);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (window), "vbox1", vbox1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

	str = g_strdup_printf( _("Send File to %s:"), ((CONTACT_PTR)contact->data)->nick );

	label3 = gtk_label_new ( str );
	g_free( str );
	gtk_widget_ref (label3);
	gtk_object_set_data_full (GTK_OBJECT (window), "label3", label3,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label3);
	gtk_box_pack_start (GTK_BOX (vbox1), label3, FALSE, FALSE, 0);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_ref (hbox1);
	gtk_object_set_data_full (GTK_OBJECT (window), "hbox1", hbox1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, TRUE, 0);

	label1 = gtk_label_new (_("File:"));
	gtk_widget_ref (label1);
	gtk_object_set_data_full (GTK_OBJECT (window), "label1", label1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label1);
	gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);

	fileentry1 = gnome_file_entry_new ("gnomeicu_sendfile", _("GnomeICU: Select File"));
	gtk_widget_ref (fileentry1);
	gtk_object_set_data_full (GTK_OBJECT (window), "fileentry1", fileentry1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (fileentry1);
	gtk_box_pack_start (GTK_BOX (hbox1), fileentry1, TRUE, TRUE, 0);

	combo_entry1 = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (fileentry1));
	gtk_widget_ref (combo_entry1);
	gtk_object_set_data_full (GTK_OBJECT (window), "combo_entry1", combo_entry1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	GTK_WIDGET_SET_FLAGS (combo_entry1, GTK_CAN_FOCUS);
	gtk_widget_show (combo_entry1);

	label2 = gtk_label_new (_("Reason:"));
	gtk_widget_ref (label2);
	gtk_object_set_data_full (GTK_OBJECT (window), "label2", label2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label2);
	gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label2), 7.45058e-09, 0.5);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (scrolledwindow1);
	gtk_object_set_data_full (GTK_OBJECT (window), "scrolledwindow1", scrolledwindow1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

	text1 = gtk_text_new (NULL, NULL);
	gtk_widget_ref (text1);
	gtk_object_set_data_full (GTK_OBJECT (window), "text1", text1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (text1);
	gtk_text_set_word_wrap( GTK_TEXT( text1 ), TRUE );
	gtk_text_set_editable( GTK_TEXT( text1 ), TRUE );
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), text1);

	dialog_action_area1 = GNOME_DIALOG (window)->action_area;
	gtk_object_set_data (GTK_OBJECT (window), "dialog_action_area1", dialog_action_area1);
	gtk_widget_show (dialog_action_area1);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (window),
	                                        _("Send"), GNOME_STOCK_PIXMAP_JUMP_TO);
	button1 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
	gtk_widget_ref (button1);
	gtk_object_set_data_full (GTK_OBJECT (window), "button1", button1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (button1);
	GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

	gtk_object_set_data( GTK_OBJECT( button1 ), "file", combo_entry1 );
	gtk_object_set_data( GTK_OBJECT( button1 ), "reason", text1 );
	gtk_object_set_data( GTK_OBJECT( button1 ), "uin",
	                     GINT_TO_POINTER( ((CONTACT_PTR)contact->data)->uin ) );

	gnome_dialog_append_button (GNOME_DIALOG (window), GNOME_STOCK_BUTTON_CLOSE);
	button2 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
	gtk_widget_ref (button2);
	gtk_object_set_data_full (GTK_OBJECT (window), "button2", button2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (button2);
	GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

	gtk_object_set_data( GTK_OBJECT( text1 ), "button", button1 );
	gtk_object_set_data( GTK_OBJECT( text1 ), "cancel", button2 );

	gtk_object_set_data( GTK_OBJECT( combo_entry1 ), "button", button1 );
	gtk_object_set_data( GTK_OBJECT( combo_entry1 ), "cancel", button2 );

	gtk_signal_connect_after( GTK_OBJECT( text1 ), "key_press_event",
	                    GTK_SIGNAL_FUNC( ctrl_enter_cb ), NULL );

	gtk_signal_connect_after( GTK_OBJECT( combo_entry1 ), "key_press_event",
	                    GTK_SIGNAL_FUNC( ctrl_enter_cb ), NULL );

	gtk_signal_connect (GTK_OBJECT (button1), "clicked",
	                    GTK_SIGNAL_FUNC (send_file_request),
	                    NULL);
	gtk_signal_connect_object (GTK_OBJECT (button1), "clicked",
	                           GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                           GTK_OBJECT( window ));
	gtk_signal_connect_object (GTK_OBJECT (button2), "clicked",
	                           GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                           GTK_OBJECT( window ));

	gtk_widget_grab_focus( combo_entry1 );
	gtk_widget_show_all( window );
}

void request_file_default( GtkWidget *widget, gpointer data )
{
	request_file( widget, "" );
}

void send_url_window( GtkWidget *widget, char *defaulturl )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );

	GtkWidget *window;
	GtkWidget *dialog_vbox2;
	GtkWidget *vbox12;
	GtkWidget *label20;
	GtkWidget *table4;
	GtkWidget *label21;
	GtkWidget *label22;
	GtkWidget *entry2;
	GtkWidget *combo_entry2;
	GtkWidget *entry3;
	GtkWidget *combo_entry3;
	GtkWidget *dialog_action_area2;
	GtkWidget *button4;
	GtkWidget *button6;

	gchar *wtitle;

	struct URLInfo *urlinfo = g_new0( struct URLInfo, 1 );

	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "send_url_window\n" );
#endif

	contact = Contacts;
	
	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	urlinfo->contact = contact;

	wtitle = g_strdup_printf( _("Send URL to: %s"), ((CONTACT_PTR)contact->data)->nick );

	window = gnome_dialog_new (_("GnomeICU: Send URL"), NULL);
	gtk_widget_set_name (window, "window");
	gtk_object_set_data (GTK_OBJECT (window), "window", window);
	gtk_window_set_default_size ( GTK_WINDOW( window ), 400, -2);
	GTK_WINDOW (window)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
	gtk_window_set_default_size( GTK_WINDOW( window ), 500, 150 );
	gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);
	gtk_window_set_wmclass (GTK_WINDOW (window), "SendURL", "GnomeICU");

	dialog_vbox2 = GNOME_DIALOG (window)->vbox;
	gtk_widget_set_name (dialog_vbox2, "dialog_vbox2");
	gtk_object_set_data (GTK_OBJECT (window), "dialog_vbox2", dialog_vbox2);
	gtk_widget_show (dialog_vbox2);

	vbox12 = gtk_vbox_new (FALSE, 4);
	gtk_widget_set_name (vbox12, "vbox12");
	gtk_widget_ref (vbox12);
	gtk_object_set_data_full (GTK_OBJECT (window), "vbox12", vbox12,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox12);
	gtk_box_pack_start (GTK_BOX (dialog_vbox2), vbox12, TRUE, TRUE, 0);

	label20 = gtk_label_new ( wtitle );
	g_free( wtitle );
	gtk_widget_set_name (label20, "label20");
	gtk_widget_ref (label20);
	gtk_object_set_data_full (GTK_OBJECT (window), "label20", label20,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label20);
	gtk_box_pack_start (GTK_BOX (vbox12), label20, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label20), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label20), 0, 0.5);

	table4 = gtk_table_new (2, 2, FALSE);
	gtk_widget_set_name (table4, "table4");
	gtk_widget_ref (table4);
	gtk_object_set_data_full (GTK_OBJECT (window), "table4", table4,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (table4);
	gtk_box_pack_start (GTK_BOX (vbox12), table4, TRUE, TRUE, 0);
	gtk_table_set_row_spacings (GTK_TABLE (table4), 4);
	gtk_table_set_col_spacings (GTK_TABLE (table4), 4);

	label21 = gtk_label_new (_("URL:"));
	gtk_widget_set_name (label21, "label21");
	gtk_widget_ref (label21);
	gtk_object_set_data_full (GTK_OBJECT (window), "label21", label21,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label21);
	gtk_table_attach (GTK_TABLE (table4), label21, 0, 1, 0, 1,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label21), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (label21), 1, 0.5);

	label22 = gtk_label_new (_("Description:"));
	gtk_widget_set_name (label22, "label22");
	gtk_widget_ref (label22);
	gtk_object_set_data_full (GTK_OBJECT (window), "label22", label22,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label22);
	gtk_table_attach (GTK_TABLE (table4), label22, 0, 1, 1, 2,
	                  (GtkAttachOptions) (GTK_FILL),
	                  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label22), GTK_JUSTIFY_RIGHT);
	gtk_misc_set_alignment (GTK_MISC (label22), 1, 0.5);

	entry2 = gnome_entry_new ("gnomeicu_url");
	gtk_widget_set_name (entry2, "entry2");
	gtk_widget_ref (entry2);
	gtk_object_set_data_full (GTK_OBJECT (window), "entry2", entry2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (entry2);
	gtk_table_attach (GTK_TABLE (table4), entry2, 1, 2, 0, 1,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);

	combo_entry2 = gnome_entry_gtk_entry (GNOME_ENTRY (entry2));
	gtk_widget_set_name (combo_entry2, "combo_entry2");
	gtk_widget_ref (combo_entry2);
	gtk_object_set_data_full (GTK_OBJECT (window), "combo_entry2", combo_entry2,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (combo_entry2);

	urlinfo->url = combo_entry2;

	entry3 = gnome_entry_new ("gnomeicu_history_desc");
	gtk_widget_set_name (entry3, "entry3");
	gtk_widget_ref (entry3);
	gtk_object_set_data_full (GTK_OBJECT (window), "entry3", entry3,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (entry3);
	gtk_table_attach (GTK_TABLE (table4), entry3, 1, 2, 1, 2,
	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
	                  (GtkAttachOptions) (0), 0, 0);

	combo_entry3 = gnome_entry_gtk_entry (GNOME_ENTRY (entry3));
	gtk_widget_set_name (combo_entry3, "combo_entry3");
	gtk_widget_ref (combo_entry3);
	gtk_object_set_data_full (GTK_OBJECT (window), "combo_entry3", combo_entry3,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (combo_entry3);

	urlinfo->desc = combo_entry3;

	dialog_action_area2 = GNOME_DIALOG (window)->action_area;
	gtk_widget_set_name (dialog_action_area2, "dialog_action_area2");
	gtk_object_set_data (GTK_OBJECT (window), "dialog_action_area2", dialog_action_area2);
	gtk_widget_show (dialog_action_area2);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area2), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area2), 8);

	gnome_dialog_append_button_with_pixmap (GNOME_DIALOG (window),
	                                        _("Send"), GNOME_STOCK_PIXMAP_JUMP_TO);
	button4 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
	gtk_widget_set_name (button4, "button4");
	gtk_widget_ref (button4);
	gtk_object_set_data_full (GTK_OBJECT (window), "button4", button4,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (button4);
	GTK_WIDGET_SET_FLAGS (button4, GTK_CAN_DEFAULT);

	gtk_signal_connect( GTK_OBJECT( button4 ), "clicked",
	                    GTK_SIGNAL_FUNC( icq_sendurl ), urlinfo );

	gtk_signal_connect_object( GTK_OBJECT( button4 ), "clicked",
	                           GTK_SIGNAL_FUNC( gtk_widget_destroy ),
	                           GTK_OBJECT( window ) );

	gnome_dialog_append_button (GNOME_DIALOG (window), GNOME_STOCK_BUTTON_CLOSE);
	button6 = g_list_last (GNOME_DIALOG (window)->buttons)->data;
	gtk_widget_set_name (button6, "button6");
	gtk_widget_ref (button6);
	gtk_object_set_data_full (GTK_OBJECT (window), "button6", button6,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (button6);
	GTK_WIDGET_SET_FLAGS (button6, GTK_CAN_DEFAULT);

	gtk_signal_connect_object( GTK_OBJECT( button6 ), "clicked",
	                           GTK_SIGNAL_FUNC( gtk_widget_destroy ),
	                           GTK_OBJECT( window ) );

	gtk_object_set_data( GTK_OBJECT( combo_entry2 ), "button", button4 );
	gtk_object_set_data( GTK_OBJECT( combo_entry2 ), "cancel", button6 );

	gtk_object_set_data( GTK_OBJECT( combo_entry3 ), "button", button4 );
	gtk_object_set_data( GTK_OBJECT( combo_entry3 ), "cancel", button6 );

	gtk_signal_connect_after( GTK_OBJECT( combo_entry2 ), "key_press_event",
	                    GTK_SIGNAL_FUNC( ctrl_enter_cb ), NULL );

	gtk_signal_connect_after( GTK_OBJECT( combo_entry3 ), "key_press_event",
	                    GTK_SIGNAL_FUNC( ctrl_enter_cb ), NULL );

	GTK_WIDGET_SET_FLAGS (combo_entry2, GTK_CAN_FOCUS);
	gtk_widget_grab_focus( combo_entry2 );

	gtk_widget_show_all( window );
}

void send_url_window_default( GtkWidget *widget, gpointer data )
{
	send_url_window( widget, "" );
}

void remove_user( GtkWidget *widget, gpointer data )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );
	GtkWidget *check = gtk_object_get_data( GTK_OBJECT( widget ), "delhist" );
	gchar *path;

	GSList *contact;

	gint method;

#ifdef TRACE_FUNCTION
	g_print( "remove_user\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	if( ((CONTACT_PTR)contact->data)->inlist == FALSE )
		method = UPDATE_NOTLIST;
	else if( ((CONTACT_PTR)contact->data)->status == STATUS_OFFLINE )
		method = UPDATE_OFFLINE;
	else
		method = UPDATE_ONLINE;

	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( check ) ) )
	{
		path = g_strdup_printf( "rm -f /%s/.icq/history/%d", getenv( "HOME" ),
		                        ((CONTACT_PTR)contact->data)->uin );
		system( path );
		g_free( path );
	}

	if( ((CONTACT_PTR)contact->data)->sok )
	{
		close( ((CONTACT_PTR)contact->data)->sok );
		g_io_channel_close( ((CONTACT_PTR)contact->data)->gioc );
		g_source_remove( ((CONTACT_PTR)contact->data)->giocw );
	}

	g_free( (CONTACT_PTR)contact->data );

	Contacts = g_slist_remove( Contacts, ((CONTACT_PTR)contact->data) );

	Save_RC();

	applet_update( Current_Status, NULL );
	Show_Quick_Status_lower( method, contact );
}

void show_info_new( int uin )
{
	static int first = TRUE;

	gchar *str[5];

#ifdef TRACE_FUNCTION
	g_print( "show_info_new\n" );
#endif

	if( first )
	{
		memset( &New_Contact, 0x00, sizeof( New_Contact ) );
		New_Contact.info = g_new0( USER_INFO_STRUCT, 1 );
		first = FALSE;
	}

	New_Contact.info->window = found_list;
	str[0] = g_strdup_printf( "%d\n", uin );
	str[1] = str[2] = str[3] = str[4] = g_strdup( "" );

	gtk_clist_append( GTK_CLIST( New_Contact.info->window ), str );
	g_free( str[0] );
	g_free( str[1] );

	if( start_new_info )
	{
		info_req_99( uin );
		info_req_99( uin );
/*		dump_personal_info( uin );*/
		start_new_info = FALSE;
	}
}

void retrieve_away_message( GtkWidget *widget, gpointer data )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );

	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "retrieve_away_message\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	((CONTACT_PTR)contact->data)->show_again = TRUE;

	TCPRetrieveAwayMessage( contact, NULL );
}

void retrieve_version( GtkWidget *widget, gpointer data )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );

	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "retrieve_away_message\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	((CONTACT_PTR)contact->data)->show_again = TRUE;

	TCPRetrieveVersion( contact, NULL );
}

void icq_set_status_online( GtkWidget *widget, gpointer data )
{
#ifdef TRACE_FUNCTION
	g_print( "icq_set_status_online\n" );
#endif

	if( Current_Status == STATUS_OFFLINE )
	{
		create_tcp_line();
		preset_status = STATUS_ONLINE;
		Done_Login = FALSE;
		MainData->sok = Connect_Remote( server, remote_port, STDERR );
		if( MainData->sok > 0 )
		{
			udp_gdk_input = gdk_input_add( MainData->sok, GDK_INPUT_READ, (GdkInputFunction) icq_refresh, NULL );
			Connected = TRUE;

			Current_Status = STATUS_ONLINE | (toggles->webpresence*0x10000);
			Login( passwd, our_ip, our_port );
		}
	}
	else
	{
		icq_change_status( STATUS_ONLINE );
	}
}

void icq_set_status_offline( GtkWidget *widget, gpointer data )
{
	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "icq_set_status_offline\n" );
#endif

	contact = Contacts;

	if( Current_Status == STATUS_OFFLINE )
		return;

	Current_Status = STATUS_OFFLINE;
	Quit_ICQ();
	if( udp_gdk_input )
	{
		gdk_input_remove( udp_gdk_input );
		udp_gdk_input = 0;
	}

	while( contact != NULL )
	{
		((CONTACT_PTR)contact->data)->last_status = ((CONTACT_PTR)contact->data)->status;
		((CONTACT_PTR)contact->data)->status = STATUS_OFFLINE;
		((CONTACT_PTR)contact->data)->have_tcp_connection = FALSE;
		if( ((CONTACT_PTR)contact->data)->sok )
		{
			g_io_channel_close( ((CONTACT_PTR)contact->data)->gioc );
			close( ((CONTACT_PTR)contact->data)->sok );
			g_source_remove( ((CONTACT_PTR)contact->data)->giocw );
			((CONTACT_PTR)contact->data)->gioc = 0;
			((CONTACT_PTR)contact->data)->sok = 0;
		}
		contact = contact->next;
	}

	Show_Quick_Status();
	seq_num = 1;

	if( MainData->sok )
		close( MainData->sok );

	Connected = FALSE;
	enable_online_events = FALSE;

	ready_set();

	if( toggles->applet )
		applet_update( Current_Status, NULL );

	msg_queue_clear();
}

int icq_set_status_away( GtkWidget *widget, gpointer data )
{
#ifdef TRACE_FUNCTION
	g_print( "icq_set_status_away\n" );
#endif

	if( Current_Status == STATUS_OFFLINE )
	{
		create_tcp_line();
		preset_status = STATUS_AWAY;
		Done_Login = FALSE;
		MainData->sok = Connect_Remote( server, remote_port, STDERR );
		if( MainData->sok > 0 )
		{
			Current_Status = STATUS_AWAY | (toggles->webpresence*0x10000);
			udp_gdk_input = gdk_input_add( MainData->sok, GDK_INPUT_READ, (GdkInputFunction) icq_refresh, NULL );
			Connected = TRUE;

			Login( passwd, our_ip, our_port );
		}
	}
	else
	{
		icq_change_status( STATUS_AWAY );
		change_away_window( NULL, NULL );
	}

	return FALSE;
}

void icq_set_status_na( GtkWidget *widget, gpointer data )
{
#ifdef TRACE_FUNCTION
	g_print( "icq_set_status_na\n" );
#endif

	if( Current_Status == STATUS_OFFLINE )
	{
		create_tcp_line();
		preset_status = STATUS_NA;
		Done_Login = FALSE;
		MainData->sok = Connect_Remote( server, remote_port, STDERR );
		if( MainData->sok > 0 )
		{
			Current_Status = STATUS_NA | (toggles->webpresence*0x10000);
			udp_gdk_input = gdk_input_add( MainData->sok, GDK_INPUT_READ, (GdkInputFunction) icq_refresh, NULL );
			Connected = TRUE;

			Login( passwd, our_ip, our_port );
		}
	}
	else
	{
		icq_change_status( STATUS_NA );
		change_away_window( NULL, NULL );
	}
}

void icq_set_status_invisible( GtkWidget *widget, gpointer data )
{
#ifdef TRACE_FUNCTION
	g_print( "icq_set_status_invisible\n" );
#endif

	if( Current_Status == STATUS_OFFLINE )
	{
		create_tcp_line();
		preset_status = STATUS_INVISIBLE;
		Done_Login = FALSE;
		MainData->sok = Connect_Remote( server, remote_port, STDERR );
		if( MainData->sok > 0 )
		{
			Current_Status = STATUS_INVISIBLE | (toggles->webpresence*0x10000);
			udp_gdk_input = gdk_input_add( MainData->sok, GDK_INPUT_READ, (GdkInputFunction) icq_refresh, NULL );
			Connected = TRUE;

			Login( passwd, our_ip, our_port );
		}
	}
	else
	{
		icq_change_status( STATUS_INVISIBLE );
	}
}

void icq_set_status_ffc( GtkWidget *widget, gpointer data )
{
#ifdef TRACE_FUNCTION
	g_print( "icq_set_status_ffc\n" );
#endif

	if( Current_Status == STATUS_OFFLINE )
	{
		create_tcp_line();
		preset_status = STATUS_FREE_CHAT;
		Done_Login = FALSE;
		MainData->sok = Connect_Remote( server, remote_port, STDERR );
		if( MainData->sok > 0 )
		{
			Current_Status = STATUS_FREE_CHAT | (toggles->webpresence*0x10000);
			udp_gdk_input = gdk_input_add( MainData->sok, GDK_INPUT_READ, (GdkInputFunction) icq_refresh, NULL );
			Connected = TRUE;

			Login( passwd, our_ip, our_port );
		}
	}
	else
	{
		icq_change_status( STATUS_FREE_CHAT );
	}
}

void icq_set_status_occ( GtkWidget *widget, gpointer data )
{
#ifdef TRACE_FUNCTION
	g_print( "icq_set_status_occ\n" );
#endif

	if( Current_Status == STATUS_OCCUPIED )
		return;

	if( Current_Status == STATUS_OFFLINE )
	{
		create_tcp_line();
		preset_status = STATUS_OCCUPIED;
		Done_Login = FALSE;
		MainData->sok = Connect_Remote( server, remote_port, STDERR );
		if( MainData->sok > 0 )
		{
			Current_Status = STATUS_OCCUPIED | (toggles->webpresence*0x10000);
			udp_gdk_input = gdk_input_add( MainData->sok, GDK_INPUT_READ, (GdkInputFunction) icq_refresh, NULL );
			Connected = TRUE;

			Login( passwd, our_ip, our_port );
		}
	}
	else
	{
		icq_change_status( STATUS_OCCUPIED );
		change_away_window( NULL, NULL );
	}
}

void icq_set_status_dnd( GtkWidget *widget, gpointer data )
{
#ifdef TRACE_FUNCTION
	g_print( "icq_set_status_dnd\n" );
#endif

	if( Current_Status == STATUS_DND )
		return;

	if( Current_Status == STATUS_OFFLINE )
	{
		create_tcp_line();
		preset_status = STATUS_DND;
		Done_Login = FALSE;
		MainData->sok = Connect_Remote( server, remote_port, STDERR );
		if( MainData->sok > 0 )
		{
			Current_Status = STATUS_DND | (toggles->webpresence*0x10000);
			udp_gdk_input = gdk_input_add( MainData->sok, GDK_INPUT_READ, (GdkInputFunction) icq_refresh, NULL );
			Connected = TRUE;

			Login( passwd, our_ip, our_port );
		}
	}
	else
	{
		icq_change_status( STATUS_DND );
		change_away_window( NULL, NULL );
	}
}

void display_url( GtkWidget *widget, char *url )
{
#ifdef TRACE_FUNCTION
	g_print( "display_url\n" );
#endif

	gnome_url_show( url );
}

void display_url_from_entry( GtkWidget *widget, GtkEntry *entry )
{
#ifdef TRACE_FUNCTION
	g_print( "display_url_from_entry\n" );
#endif
	display_url( NULL, gtk_entry_get_text( entry ) );
}

void add_from_search( GtkWidget *widget, gpointer data )
{
	int new_uin;
	int row;
	char name[16];

#ifdef TRACE_FUNCTION
	g_print( "add_from_search\n" );
#endif

	row = GPOINTER_TO_INT( GTK_CLIST( found_list )->selection->data );
	new_uin = GPOINTER_TO_INT(
		gtk_clist_get_row_data( GTK_CLIST( found_list ), row ) );
	sprintf( name, "%d", new_uin );
	Add_User( new_uin, name, TRUE );
}

gint app_save_state_cb( GnomeClient *client, gint phase, GnomeRestartStyle save_style, gint shutdown, GnomeInteractStyle interact_style, gint fast, gpointer client_data )
{
	gchar *prefix= gnome_client_get_config_prefix (client);
	gchar *argv[]= { "rm", "-r", NULL, NULL };
	gchar *config_param = NULL;
	int i = 1;

#ifdef TRACE_FUNCTION
	g_print ("app_save_state_cb\n");
#endif

	/* Here is the real SM code. We set the argv to the parameters needed
	   to restart/discard the session that we've just saved and call
	   the gnome_session_set_*_command to tell the session manager it. */

	argv[2] = gnome_config_get_real_path (prefix);
	gnome_client_set_discard_command (client, 3, argv);

	/* Set commands to clone and restart this application.  Note that we
	   use the same values for both -- the session management code will
	   automatically add whatever magic option is required to set the
	   session id on startup.  */

	argv[0]= (gchar*) client_data;
	if ( toggles->applet == FALSE )
		argv[i++] = "-a";
	if ( strlen( configfilename ) )
	{
		config_param = g_strdup_printf ("--config=%s", configfilename);
		argv[i++] = config_param;
	}

	gnome_client_set_restart_command (client, i, argv);
	gnome_client_set_clone_command (client, 0, NULL);

	if ( config_param )
		g_free(config_param);

	return TRUE;
}

int hide_ch_window( GtkWidget *widget, GdkEvent *event, GtkWidget *window )
{
#ifdef TRACE_FUNCTION
	g_print( "hide_ch_window\n" );
#endif
	gtk_widget_hide( window );
	return TRUE;
}

/* Callback for status menu positioning */
static void cb_menu_position (GtkMenu *menu, gint *x, gint *y, gpointer user_data)
{
	GtkWidget *widget, *button;
	gint screen_width, screen_height;

	GtkRequisition requisition;

	widget = GTK_WIDGET (menu);
	button = GTK_WIDGET (user_data);

	gtk_widget_size_request (widget, &requisition);

	gdk_window_get_origin (button->window, x, y);
	*y -= requisition.height;

	screen_width = gdk_screen_width ();
	screen_height = gdk_screen_height ();

	if (*x + requisition.width > screen_width) {
	  *x -= *x + requisition.width - screen_width;
	}
}

int popup_status_menu( GtkWidget *widget, gpointer data )
{
	GtkWidget *status_menu;
	GtkWidget *pm, *item_box, *item_label;
	GtkWidget *item;

	status_menu = gtk_menu_new();

	item_box = gtk_hbox_new( FALSE, 0 );
	pm = gtk_pixmap_new( icon_online_pixmap, icon_online_bitmap );
	gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
	gtk_widget_show( pm );
	item = gtk_menu_item_new();
	item_label = gtk_label_new( _("  Online") );
	gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
	gtk_widget_show( item_label );
	gtk_container_add( GTK_CONTAINER( item ), item_box );
	gtk_widget_show( item_box );
	gtk_menu_append( GTK_MENU( status_menu ), item );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
	                    GTK_SIGNAL_FUNC( icq_set_status_online ),
	                    NULL );
	gtk_widget_show( item );

	item_box = gtk_hbox_new( FALSE, 0 );
	pm = gtk_pixmap_new( icon_away_pixmap, icon_away_bitmap );
	gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
	gtk_widget_show( pm );
	item = gtk_menu_item_new();
	item_label = gtk_label_new( _("  Away") );
	gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
	gtk_widget_show( item_label );
	gtk_container_add( GTK_CONTAINER( item ), item_box );
	gtk_widget_show( item_box );
	gtk_menu_append( GTK_MENU( status_menu ), item );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
	                    GTK_SIGNAL_FUNC( icq_set_status_away ),
	                    NULL );
	gtk_widget_show( item );

	item_box = gtk_hbox_new( FALSE, 0 );
	pm = gtk_pixmap_new( icon_na_pixmap, icon_na_bitmap );
	gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
	gtk_widget_show( pm );
	item = gtk_menu_item_new();
	item_label = gtk_label_new( _("  Not Available") );
	gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
	gtk_widget_show( item_label );
	gtk_container_add( GTK_CONTAINER( item ), item_box );
	gtk_widget_show( item_box );
	gtk_menu_append( GTK_MENU( status_menu ), item );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
	                    GTK_SIGNAL_FUNC( icq_set_status_na ),
	                    NULL );
	gtk_widget_show( item );

	item_box = gtk_hbox_new( FALSE, 0 );
	pm = gtk_pixmap_new( icon_ffc_pixmap, icon_ffc_bitmap );
	gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
	gtk_widget_show( pm );
	item = gtk_menu_item_new();
	item_label = gtk_label_new( _("  Free for Chat") );
	gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
	gtk_widget_show( item_label );
	gtk_container_add( GTK_CONTAINER( item ), item_box );
	gtk_widget_show( item_box );
	gtk_menu_append( GTK_MENU( status_menu ), item );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
	                    GTK_SIGNAL_FUNC( icq_set_status_ffc ),
	                    NULL );
	gtk_widget_show( item );

	item_box = gtk_hbox_new( FALSE, 0 );
	pm = gtk_pixmap_new( icon_occ_pixmap, icon_occ_bitmap );
	gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
	gtk_widget_show( pm );
	item = gtk_menu_item_new();
	item_label = gtk_label_new( _("  Occupied") );
	gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
	gtk_widget_show( item_label );
	gtk_container_add( GTK_CONTAINER( item ), item_box );
	gtk_widget_show( item_box );
	gtk_menu_append( GTK_MENU( status_menu ), item );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
	                    GTK_SIGNAL_FUNC( icq_set_status_occ ),
	                    NULL );
	gtk_widget_show( item );

	item_box = gtk_hbox_new( FALSE, 0 );
	pm = gtk_pixmap_new( icon_dnd_pixmap, icon_dnd_bitmap );
	gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
	gtk_widget_show( pm );
	item = gtk_menu_item_new();
	item_label = gtk_label_new( _("  Do not Disturb") );
	gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
	gtk_widget_show( item_label );
	gtk_container_add( GTK_CONTAINER( item ), item_box );
	gtk_widget_show( item_box );
	gtk_menu_append( GTK_MENU( status_menu ), item );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
	                    GTK_SIGNAL_FUNC( icq_set_status_dnd ),
	                    NULL );
	gtk_widget_show( item );

	item_box = gtk_hbox_new( FALSE, 0 );
	pm = gtk_pixmap_new( icon_inv_pixmap, icon_inv_bitmap );
	gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
	gtk_widget_show( pm );
	item = gtk_menu_item_new();
	item_label = gtk_label_new( _("  Invisible") );
	gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
	gtk_widget_show( item_label );
	gtk_container_add( GTK_CONTAINER( item ), item_box );
	gtk_widget_show( item_box );
	gtk_menu_append( GTK_MENU( status_menu ), item );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
	                    GTK_SIGNAL_FUNC( icq_set_status_invisible ),
	                    NULL );
	gtk_widget_show( item );

	item = gtk_menu_item_new();
	gtk_widget_set_sensitive( item, FALSE );
	gtk_menu_append( GTK_MENU( status_menu ), item );
	gtk_widget_show( item );

	item_box = gtk_hbox_new( FALSE, 0 );
	pm = gtk_pixmap_new( icon_offline_pixmap, icon_offline_bitmap );
	gtk_box_pack_start( GTK_BOX( item_box ), pm, FALSE, FALSE, 0 );
	gtk_widget_show( pm );
	item = gtk_menu_item_new();
	item_label = gtk_label_new( _("  Offline") );
	gtk_box_pack_start( GTK_BOX( item_box ), item_label, FALSE, FALSE, 0 );
	gtk_widget_show( item_label );
	gtk_container_add( GTK_CONTAINER( item ), item_box );
	gtk_widget_show( item_box );
	gtk_menu_append( GTK_MENU( status_menu ), item );
	gtk_signal_connect( GTK_OBJECT( item ), "activate",
	                    GTK_SIGNAL_FUNC( icq_set_status_offline ),
	                    NULL );
	gtk_widget_show( item );

	gtk_widget_show( status_menu );

	gtk_menu_popup( GTK_MENU( status_menu ), NULL, NULL, 
	                cb_menu_position, widget,
	                0, 0 );

	return TRUE;
}

gint ctrl_enter_cb( GtkWidget *widget, GdkEventKey *ev, int sock )
{
	switch( ev->keyval )
	{
		case GDK_Return:
			if( ev->state & GDK_CONTROL_MASK &&
			    gtk_object_get_data( GTK_OBJECT( widget ), "button" ) != NULL )
				gtk_signal_emit_by_name( GTK_OBJECT( gtk_object_get_data( GTK_OBJECT( widget ), "button" ) ), "clicked", MainData );
			break;
		case 'c':
			if( ev->state & GDK_CONTROL_MASK &&
			    gtk_object_get_data( GTK_OBJECT( widget ), "cancel" ) != NULL )
				gtk_signal_emit_by_name( GTK_OBJECT( gtk_object_get_data( GTK_OBJECT( widget ), "cancel" ) ), "clicked", MainData );
			break;
		case 'n':
			if( ev->state & GDK_CONTROL_MASK &&
			    gtk_object_get_data( GTK_OBJECT( widget ), "readnext" ) != NULL )
				gtk_signal_emit_by_name( GTK_OBJECT( gtk_object_get_data( GTK_OBJECT( widget ), "readnext" ) ), "clicked", MainData );
			break;
	}
	
	return TRUE;
}



/*** Local functions ***/
void icq_allow_auth( GtkWidget *widget, gpointer data )
{
#ifdef TRACE_FUNCTION
	g_print( "icq_allow_auth\n" );
#endif

	icq_sendauthmsg( GPOINTER_TO_INT( gtk_object_get_data( GTK_OBJECT( widget ), "uin" ) ) );
}

void icq_accept_chat( GtkWidget *widget, GSList *contact )
{
	int sock = ((CONTACT_PTR)contact->data)->sok;
	DWORD seq = ((CONTACT_PTR)contact->data)->chat_seq;

#ifdef TRACE_FUNCTION
	g_print( "icq_accept_chat\n" );
#endif

	TCPAcceptChat( sock, contact, seq );
}

void icq_refuse_chat( GtkWidget *widget, GSList *contact )
{
	int sock = ((CONTACT_PTR)contact->data)->sok;
	DWORD seq = ((CONTACT_PTR)contact->data)->chat_seq;

#ifdef TRACE_FUNCTION
	g_print( "icq_refuse_chat\n" );
#endif

	TCPRefuseChat( sock, seq );
}

void icq_accept_file( GtkWidget *widget, GSList *contact )
{
	int sock = ((CONTACT_PTR)contact->data)->sok;
	DWORD seq = ((CONTACT_PTR)contact->data)->file_seq;

#ifdef TRACE_FUNCTION
	g_print( "icq_accept_file\n" );
#endif

	TCPAcceptFile( sock, contact, seq );
}

void icq_refuse_file( GtkWidget *widget, GSList *contact )
{
	int sock = ((CONTACT_PTR)contact->data)->sok;
	DWORD seq = ((CONTACT_PTR)contact->data)->file_seq;

#ifdef TRACE_FUNCTION
	g_print( "icq_refuse_file\n" );
#endif

	TCPRefuseFile( sock, seq );
}

void icq_refresh_list( GtkWidget *widget, gpointer data )
{
#ifdef TRACE_FUNCTION
	g_print( "icq_refresh_list\n" );
#endif

	Show_Quick_Status();
}

void clear_i_window( GtkWidget *wigdet, gpointer data )
{
	int uin;

	GSList *contact;
	
	uin = GPOINTER_TO_INT( data );

	contact = Contacts;

	while( contact != NULL )
	{
		if( ((CONTACT_PTR)contact->data)->uin == uin )
			break;
		contact = contact->next;
	}
	
	if( contact != NULL )
		((CONTACT_PTR)contact->data)->info->window = NULL;

	if( New_Contact.uin == uin )
		New_Contact.info->window = NULL;
}

void toggle_value( GtkWidget *widget, int *val )
{
	GSList *contact;

	*val = ! *val;

	contact = Contacts;
	while( contact != NULL )
	{
		if( &((CONTACT_PTR)contact->data)->invis_list == val && *val )
		{
			((CONTACT_PTR)contact->data)->vis_list = FALSE;
			snd_vis_list();
		}
		if( &((CONTACT_PTR)contact->data)->vis_list == val && *val )
		{
			((CONTACT_PTR)contact->data)->invis_list = FALSE;
			snd_invis_list();
		}
		contact = contact->next;
	}
}

void grab_history( GtkWidget *widget, gpointer data )
{
	GSList *contact = gtk_object_get_data( GTK_OBJECT( gtk_menu_get_active( gtk_object_get_data( GTK_OBJECT( widget ), "menu" ) ) ), "contact" );
	display_mess_history( NULL, contact );
}

void get_contact_list( GtkWidget *widget, gpointer data )
{
	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );
	GtkWidget *dialog;

	GSList *contact;

#ifdef TRACE_FUNCTION
	g_print( "request_chat\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	dialog = create_contact_list_window( ((CONTACT_PTR)contact->data)->uin );
	gtk_window_set_title(GTK_WINDOW( dialog ), _("GnomeICU: Send Contact List"));
	gtk_widget_show( dialog );
}

void remove_user_question( GtkWidget *widget, gpointer data )
{
	GtkWidget *rmcontact_dialog;
	GtkWidget *dialog_vbox1;
	GtkWidget *vbox1;
	GtkWidget *label;
	GtkWidget *remhist_button;
	GtkWidget *dialog_action_area1;
	GtkWidget *yes_button;
	GtkWidget *no_button;

	GtkCList *clist = GTK_CLIST( MainData->lb_userwin );

	GSList *contact;
	gchar *str;

#ifdef TRACE_FUNCTION
	g_print( "remove_user\n" );
#endif

	contact = Contacts;

	while( contact != NULL )
	{
		if( GPOINTER_TO_INT( gtk_clist_get_row_data( GTK_CLIST( clist ), GPOINTER_TO_INT( clist->selection->data ) ) ) ==
		    ((CONTACT_PTR)contact->data)->uin )
			break;
		contact = contact->next;
	}

	if( contact == NULL )
		return;

	rmcontact_dialog = gnome_dialog_new (_("GnomeICU: Remove Contact"), NULL);
	gtk_object_set_data (GTK_OBJECT (rmcontact_dialog), "rmcontact_dialog", rmcontact_dialog);
	gtk_window_set_modal (GTK_WINDOW (rmcontact_dialog), TRUE);
	gtk_window_set_policy (GTK_WINDOW (rmcontact_dialog), FALSE, FALSE, FALSE);

	dialog_vbox1 = GNOME_DIALOG (rmcontact_dialog)->vbox;
	gtk_object_set_data (GTK_OBJECT (rmcontact_dialog), "dialog_vbox1", dialog_vbox1);
	gtk_widget_show (dialog_vbox1);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (rmcontact_dialog), "vbox1", vbox1,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

	str = g_strdup_printf( _("Are you sure you wish to remove\n%s from your contact list?\n"),
	                       ((CONTACT_PTR)contact->data)->nick );
	label = gtk_label_new ( str );
	g_free( str );
	gtk_widget_ref (label);
	gtk_object_set_data_full (GTK_OBJECT (rmcontact_dialog), "label", label,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (vbox1), label, FALSE, FALSE, 0);

	remhist_button = gtk_check_button_new_with_label (_("Remove User's History file"));
	gtk_widget_ref (remhist_button);
	gtk_object_set_data_full (GTK_OBJECT (rmcontact_dialog), "remhist_button", remhist_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (remhist_button);
	gtk_box_pack_start (GTK_BOX (vbox1), remhist_button, FALSE, FALSE, 0);

	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( remhist_button ), TRUE );

	dialog_action_area1 = GNOME_DIALOG (rmcontact_dialog)->action_area;
	gtk_object_set_data (GTK_OBJECT (rmcontact_dialog), "dialog_action_area1", dialog_action_area1);
	gtk_widget_show (dialog_action_area1);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

	gnome_dialog_append_button (GNOME_DIALOG (rmcontact_dialog), GNOME_STOCK_BUTTON_YES);
	yes_button = g_list_last (GNOME_DIALOG (rmcontact_dialog)->buttons)->data;
	gtk_widget_ref (yes_button);
	gtk_object_set_data_full (GTK_OBJECT (rmcontact_dialog), "yes_button", yes_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (yes_button);
	GTK_WIDGET_SET_FLAGS (yes_button, GTK_CAN_DEFAULT);

	gnome_dialog_append_button (GNOME_DIALOG (rmcontact_dialog), GNOME_STOCK_BUTTON_NO);
	no_button = g_list_last (GNOME_DIALOG (rmcontact_dialog)->buttons)->data;
	gtk_widget_ref (no_button);
	gtk_object_set_data_full (GTK_OBJECT (rmcontact_dialog), "no_button", no_button,
	                          (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (no_button);
	GTK_WIDGET_SET_FLAGS (no_button, GTK_CAN_DEFAULT);

	gtk_object_set_data( GTK_OBJECT( yes_button ), "delhist", remhist_button );

	gtk_signal_connect (GTK_OBJECT (yes_button), "clicked",
	                    GTK_SIGNAL_FUNC (remove_user),
	                    NULL);
	gtk_signal_connect_object( GTK_OBJECT (yes_button), "clicked",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    GTK_OBJECT( rmcontact_dialog ));
	gtk_signal_connect_object( GTK_OBJECT (no_button), "clicked",
	                    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	                    GTK_OBJECT( rmcontact_dialog ));

	gtk_widget_show_all( rmcontact_dialog );
}

void begin_search( GnomeDruidPage *page, GnomeDruid *druid )
{
	UIN_T uin;
	gchar *nick, *first, *last, *email;
	GtkEntry *entry;

#ifdef TRACE_FUNCTION
	g_print( "begin_search\n" );
#endif

	entry = gtk_object_get_data( GTK_OBJECT( page ), "search_uin" );
	uin = atoi( gtk_entry_get_text( entry ) );
	entry = gtk_object_get_data( GTK_OBJECT( page ), "search_nick" );
	nick = gtk_entry_get_text( entry );
	entry = gtk_object_get_data( GTK_OBJECT( page ), "search_first" );
	first = gtk_entry_get_text( entry );
	entry = gtk_object_get_data( GTK_OBJECT( page ), "search_last" );
	last = gtk_entry_get_text( entry );
	entry = gtk_object_get_data( GTK_OBJECT( page ), "search_email" );
	email = gtk_entry_get_text( entry );

	found_list = GTK_WIDGET( gtk_object_get_data( GTK_OBJECT( page ), "found_list" ) );
	gtk_clist_clear( GTK_CLIST( found_list ) );

	start_search( email, nick, first, last, uin );
}

void reply_button_cb( GtkWidget *widget, gpointer data )
{
	GtkWidget *box = GTK_WIDGET( gtk_object_get_data( GTK_OBJECT( widget ), "box" ) );
	GtkWidget *hbox = GTK_WIDGET( gtk_object_get_data( GTK_OBJECT( widget ), "hbox" ) );
	GtkWidget *text = GTK_WIDGET( gtk_object_get_data( GTK_OBJECT( widget ), "text" ) );
	GtkWidget *reply = GTK_WIDGET( gtk_object_get_data( GTK_OBJECT( widget ), "reply" ) );

	gtk_widget_show( reply );
	gtk_widget_hide_all( hbox );
	gtk_widget_show( box );
	gtk_widget_grab_focus( text );
}

gchar *stripr( gchar *str )
{
	gchar *new;
	gint cx = 0, cy = 0;

	new = (gchar*)g_malloc0( strlen( str ) + 1 );

	while( str[cx] )
	{
		if( str[cx] != '\r' )
			new[cy++] = str[cx];
		cx++;
	}

	return new;
}
